//
// "$Id: $"
//
// Document. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "global.h"
#include "Document.h"
#include "PrefDialog.h"
#include "TheApp.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_PNG_Image.H>

#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>


Document::Document(int X, int Y, int W, int H, char* pa) : PrintDocument (X, Y, W, H)
{
	if (pa) {
		path_ = strdup(pa);
	} else {
		path_ = NULL;
	}
	changed_ = 0;

	tabs = new Fl_Tabs (X, Y, W, H);
	tabs->tab_height(25);
	
	Y += 25;


	customer = new Customer(X + 5, Y + 5, W - 10, H - 35);

	header = new Header(X + 5, Y + 5, W - 10, H - 35);

	position = new Position(X + 5, Y + 5, W - 10, H - 35);

	summary = new Summary(X + 5, Y + 5, W - 10, H - 35);

	footer = new Footer(X + 5, Y + 5, W - 10, H - 35);
	
	Fl_Group *grp = new Fl_Group(X + 5, Y, W - 10, H - 30);
	pagesetup = new PageSetup(X + 5, Y + 5, W - 10, H - 35);
	grp->end();

	tabs->end();

	res = new Fl_Box(X + W - 12, Y + H - 32, 2, 2);
	res->hide();

	resizable(res);

	company = new Company(this);

	if (pa) {
		load();
	} else {
		init();
	}
	
}

Document::~Document()
{
	char buf[2048];
	
	delete (company);

	snprintf(buf, 2000, "%s.tmp", path_); 
	sqlite3_close(db);
	fl_unlink(buf);

	free(path_);
}

void Document::save()
{
	FILE *in, *out;
	int err;
	char buf[2048];
	char b[1024];
	
	snprintf(buf, 2000, "%s.tmp", path_); 
	sqlite3_close(db);

	// copy
	in = fl_fopen(buf, "rb");
	out = fl_fopen(path_, "wb");
	while (!feof(in)) {
		int l;
		l = fread(b, 1, 1024, in);
		fwrite(b, 1, l, out);
	}
	fclose(in);
	fclose(out);

	err = sqlite3_open(buf, &db);
	if (!db) {
		fl_alert(_("Re-Ouverture de fichier impossible\n"));
		exit(-1);
		return;
	}
	changed_ = 0;
	pagesetup->changed_ = 0;
	customer->changed_ = 0;
	header->changed_ = 0;
	footer->changed_ = 0;
	position->changed_ = 0;
	summary->changed_ = 0;
}

Fl_PNG_Image im("c:/temp/pngtest.png");
 
void Document::print(PrintContext *ctx)
{
	ctx->x = 0;
	ctx->y = 0;

	ctx->foot_height = 20;
	ctx->head_height = 0;
	ctx->real_height = ctx->user_height - ctx->head_height - ctx->foot_height;

	ctx->page_num = ctx->from_page;
	ctx->start_page();

	company->print(ctx); // draw before clip, we draw the logo outside 
				// the margins
	ctx->tk->clip(0, 0, ctx->user_width, ctx->user_height);
	
	customer->print(ctx);	
	header->print(ctx);
	position->print(ctx);
	//summary->print(ctx);
	footer->print(ctx);	

	ctx->tk->pop_clip();

	ctx->end_page();
	
}
void Document::foot_head(PrintContext *ctx)
{
	char buf[1024];
	char page[128];
	char nb_page[128];
	const char *p1;
	char *p2;
	int w, l;

	if (pagesetup->changed_ != last_changed_pagesetup) {
		last_changed_pagesetup = pagesetup->changed_;
		pagesetup->get_font("invoice_footer_font", &ffont, &fsize, &funder);
		foot = pagesetup->get_string("invoice_footer");
	
	}
	snprintf(page, 128, "%d", ctx->page_num);
	snprintf(nb_page, 128, "%d", ctx->nb_page);
	
	p1 = foot;
	p2 = buf;
	while (*p1 && p2 < buf + 1000) {
		if (*p1 == '%') {
			p1++;
			if (*p1 == 'p') {
				snprintf(p2, 1000 - (p2 - buf), "%d", ctx->page_num);
			} else if (*p1 == 'n') {
				snprintf(p2, 1000 - (p2 - buf), "%d", ctx->nb_page);
			} else {
				snprintf(p2, 1000 - (p2 - buf), "%%%c", *p1);
			}
			p2 = buf + strlen(buf);
		} else {
			*p2 = *p1;
			p2++;
		}
		p1++;
	}
	*p2 = 0;
	fl_color(FL_BLACK);
	fl_font(ffont, fsize);
	l = strlen(buf);
	w = (int)fl_width(buf, l);
	ctx->tk->draw(buf, l, ctx->user_width / 2 - w / 2, 
		ctx->user_height + (int)fl_height());
	
}

void Document::margins(int w, int h, int left, int top, int right, 
	int bottom, int land)
{
	the_app->settings->app->set("margin_left", left);
	the_app->settings->app->set("margin_top", top);
	the_app->settings->app->set("margin_right", right);
	the_app->settings->app->set("margin_bottom", bottom);
	the_app->settings->app->set("page_width", w);
	the_app->settings->app->set("page_height", h);
	the_app->settings->app->set("landscape", land);

}

void Document::load()
{
	FILE *in, *out;
	int err = 0;
	char buf[2048];
	char b[1024];
	
	snprintf(buf, 2000, "%s.tmp", path_); 

	// copy
	in = fl_fopen(path_, "rb");
	out = fl_fopen(buf, "wb");
	while (!feof(in)) {
		int l;
		l = fread(b, 1, 1024, in);
		fwrite(b, 1, l, out);
	}
	fclose(in);
	fclose(out);

	err = sqlite3_open(buf, &db);
	if (!db) {
		fl_alert(_("Ouverture de fichier impossible\n"));
		init();
		return;
	}
	customer->load();
	header->load();
	position->load();
	summary->load();
	footer->load();
	pagesetup->load();
}

void Document::init()
{
	int err;
	char buf[2048];

	build_file_name();
	snprintf(buf, 2000, "%s.tmp", path_); 
	err = sqlite3_open(buf, &db);
	if (!db) {
		fl_alert(_("Cr\303\251ation de fichier impossible\n"
			"Verifiez vos droit d'acc\303\250s."));
		window()->hide();
		return;
	}
	customer->init();
	header->init();
	position->init();
	summary->init();
	footer->init();
	pagesetup->init();
}

void Document::build_file_name()
{
	char buf[2049];
	char dt[256];
	int month;
	char ye[256];
	char *year;
	int i;
	int num = 0;
	time_t ltime;
	struct tm *today;
	char b2[256];
	int len;
	char *invoice;
	char *letter;
	char *ext;

#ifdef INVOICE
	invoice = _("factures");
	letter = _("F");
	ext = "oki";
#else
	invoice = _("devis");
	letter = _("D");
	ext = "oke";
#endif 

	_tzset();
	time( &ltime );
	today = localtime( &ltime );
	strftime(dt, 255, "%m", today);
	month = atol(dt);
	strftime(ye, 255, "%Y", today);
	year = ye;
	if (year[0] == '2') {
		year++;
		if (year[0] == '0') year++;
	}

	snprintf(b2, 255, "%s%s-", letter, year);
	len = strlen(b2);
	for (i = month; i > 0 && num < 1; i--) {
		dirent **dir = NULL;;
		int nbd;
		
		_snprintf(buf, 2048, "%s/%s/%s/%0.2d/", 
			the_app->settings->data_path,
			invoice, ye, i);
		
		nbd = fl_filename_list(buf, &dir, fl_casenumericsort);   	
		while (nbd > 0) {
			nbd--;
			if (num < 1 && !strnicmp(dir[nbd]->d_name,  b2, len)) {
				num = atol(dir[nbd]->d_name  + len);
			} 
			free(dir[nbd]);
		}
		free(dir);
	}	
	num += 1;

	snprintf(buf, 2048, "%s/%s/%s/%s/%s%s-%0.5d.%s", 
		the_app->settings->data_path,
		invoice, ye,dt, letter, year, num, ext);
	
	fl_make_path_for_file(buf);
	free(path_);
	path_ = strdup(buf);
}

//
// End of "$Id: $".
//

