//
// "$Id: $"
//
// Application main object. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//

#ifndef DOCUMENT_H
#define DOCUMENT_H

#include <FL/Fl_Window.H>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Box.H>
#include "Customer.h"
#include "Header.h"
#include "Position.h"
#include "Printing.h"
#include "Summary.h"
#include "PageSetup.h"
#include "Footer.h"
#include "Company.h"
#include <sqlite3.h>

class Document : public PrintDocument {
public:
	sqlite3 *db;
	char *path_;
	int changed_;
	Fl_Tabs *tabs;
	Customer *customer;
	Header *header;
	Position *position;
	Summary *summary;
	Footer *footer;
	PageSetup *pagesetup;
	Company *company;

	int last_changed_pagesetup;
	Fl_Font ffont;
	int fsize;
	int funder;
	const char *foot;

	Fl_Box *res;

	Document(int X, int Y, int W, int H, char *path = NULL);
	~Document(void);

	char *path(void) { return path_;}
	int changed(void) {return changed_;}
	void clear_changed(void) {changed_ = 0;};
	void load(void);
	void init(void);
	void build_file_name(void);
	void save(void);
	void print(PrintContext *ctx);
	void foot_head(PrintContext *ctx);
	void margins(int w, int h, int left, int top, int right, int bottom, int land);

};

#endif // DOCUMENT_H

//
// End of "$Id: $".
//

