//
// "$Id: $"
//
// Customer. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "Footer.h"
#include "global.h"
#include "Document.h"
#include "TheApp.h"
#include <FL/Fl_File_Chooser.H>

#define NAME_COL 2
#define TYPE_COL 3


Footer::Footer(int X, int Y, int W, int H) :
	 Fl_Table(X, Y, W, H)
{
	int n, i;
	changed_ = 0;
	

	label(_("Fin doc."));
	labelfont(FL_HELVETICA_BOLD);
	input = new TableInput();
	input->callback(input_callback1, this);

	last_changed_pagesetup = -1;

	row_edit = 0;
	col_edit = 0;
	col_names[0] = _("Champ");
	col_names[1] = _("Valeur");
	col_width(0, 120);
	col_width(1, 470);
	
	callback(&event_callback1, (void*)this);
	when(FL_WHEN_NOT_CHANGED|when());
	rows(12);
	cols(2);
	col_header(1);
	col_resize(1);
	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll
	end();
	
	n = rows();
	values = (char***) malloc(sizeof(char**) * n);
	while (n > 0) {
		int m = cols() + 2;
		n--;
		values[n] = (char**) malloc(sizeof(char*) * m);
		while (m > 0) {
			m--;
			values[n][m] = NULL;
		}
	}

	i = 0;
	i++;
	values[i][0] = _("Saut de page");
	values[i][NAME_COL] = "foot_com1_break";
	values[i][TYPE_COL] = TBLI_CHECK;
	i++;
	values[i][0] = _("Commentaire 1");
	values[i][NAME_COL] = "foot_com1";
	values[i][TYPE_COL] = TBLI_TEXT;
	row_height(i, 100);
	i++;
	values[i][0] = _("Saut de page");
	values[i][NAME_COL] = "foot_com2_break";
	values[i][TYPE_COL] = TBLI_CHECK;
	i++;	
	values[i][0] = _("Commentaire 2");
	values[i][NAME_COL] = "foot_com2";
	values[i][TYPE_COL] = TBLI_TEXT;
	row_height(i, 100);
	i++;

}

Footer::~Footer()
{
	int n;
	n = rows();
	while (n > 0) {
		int m = cols() + 2;
		n--;
		while (m > 0) {
			m--;
			if (m == 1) {
				// free only the value entered by the user
				free(values[n][m]);
			}
		}
		free(values[n]);
	}
	free(values);
}


void Footer::event_callback1(Fl_Widget *w, void *d)
{
	((Footer*)d)->event_callback();
}

void Footer::input_callback1(Fl_Widget *w, void *d)
{
	((Footer*)d)->input_callback();
}

void Footer::input_callback()
{
	int w, h;
	w = col_width(col_edit) - 6;
	h = row_height(row_edit) - 6;
	free(values[row_edit][col_edit]);
	
	values[row_edit][col_edit] = NULL;

	if (input->value() && input->value()[0])
			values[row_edit][col_edit] = strdup(input->value());

	input->hide();
	take_focus();

	set(row_edit);
}

void Footer::get_value(char* name, char** value, char** type)
{
	int n = rows();

	*type = NULL;
	*value = NULL;
	while (n > 0) {
		n--;
		if (values[n][NAME_COL] && !strcmp(name, values[n][NAME_COL])) {
			*type = values[n][TYPE_COL];
			*value = values[n][1]; 
			return;
		}
	}
}



char *Footer::get_string(char *name)
{
	char *value;
	char *type;
	get_value(name, &value, &type);
	return value;
}

static int sql_no_cb(void *data, int argc, char **argv, char **azColName){
  Footer *p = (Footer*)data;
  return 0;
}

void Footer::set(int row)
{
	char *buf;
	int l;
	char *err = NULL;
		
	sqlite3 *db = ((Document*)parent()->parent())->db;

	if (!values[row][NAME_COL]) return;

	((Document*)parent()->parent())->changed_++;
	changed_++;

	
	if (!values[row][1]) {
		l = 1024;
		buf = (char*) malloc(1024);
		snprintf(buf, 1000, 
			"UPDATE footer SET value = '' WHERE  "
			 "key = '%s';", values[row][NAME_COL]);
		//the_app->settings->app->set(values[row][NAME_COL], "");
	} else {
		int l2;
		l = strlen(values[row][1]);
		buf = (char*) malloc(l * 2 + 1024);
		snprintf(buf, 1000, 
			"UPDATE footer SET value = '");
		l2 = strlen(buf);
		l2 += sqlite3_encode_binary((uchar*)values[row][1], l, (uchar*)buf + l2);
		snprintf(buf + l2, 1000 + l * 2 - l2, "' WHERE "
			"key = '%s';", values[row][NAME_COL]);
		//the_app->settings->app->set(values[row][NAME_COL], values[row][1]);	
	}

	if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
		fl_alert(_("Erreur SQL : %s."), err);
	}
	free(buf);
}


void Footer::event_callback()
{
    int R = callback_row(),
        C = callback_col();
    TableContext context = callback_context();

    switch ( context )
    {
	case CONTEXT_CELL:
	{
            switch (Fl::event()) 
	    { 
		case FL_PUSH:
		    row_edit = R;
		    col_edit = C;
		    if (!Fl::event_clicks()) 
		    {
			if (input->visible()) input->run_callback();
			input->hide();
			return;
		    }
		    Fl::event_clicks(0);
		    //FALLTHROUGH

		case FL_KEYBOARD:
		    row_edit = R;
		    col_edit = C;

		 
		    if (Fl::event() == FL_KEYBOARD && Fl::e_length == 0)
		        { return; }   
		    if (input->visible()) {
				input->run_callback();
		    }
		    
		    //if (R > rows() - 3) add_rows();
		    if (C < 1) {
			input->hide();
			return;
		    }	
		    set_selection(R, C, R, C);

		    int XX,YY,WW,HH;
		    find_cell(CONTEXT_CELL, R, C, XX, YY, WW, HH);
		    input->resize(XX,YY,WW,HH);
		    input->value(values[R][C]);
		    input->show(values[R][TYPE_COL]);
		    if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') 
			{ input->handle(Fl::event()); }
		    return;
	    }
	    return;
	}

	case CONTEXT_ROW_HEADER:
	case CONTEXT_COL_HEADER:
	    if (input->visible()) input->run_callback();
            input->hide();
	    return;

	case CONTEXT_TABLE:
	    if (R < 0 && C < 0)
	    {
	        if (input->visible()) input->run_callback();
                input->hide();
	    }
	    return;
	
	default:
	    return;
    }
}

void Footer::draw_cell(TableContext context,
	int R, int C, int X, int Y, int W, int H)
{

    switch ( context )
    {
	case CONTEXT_STARTPAGE:
	    // Get kb nav + mouse 'selection region' for use below
	    get_selection(s_top, s_left, s_bottom, s_right);
	    s_top = s_bottom;
	    s_left = s_right; 	
	    break;

	case CONTEXT_COL_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, col_header_color());
		fl_color(FL_BLACK);
		fl_draw(col_names[C],  X, Y, W, H, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_ROW_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{
	    if (R == row_edit && C == col_edit && input->visible())
	        { return; }

	    if (R >= s_top && R <= s_bottom && C >= s_left && C <= s_right) { 
		input->draw_selected(X, Y, W, H, values[R][C], values[R][TYPE_COL]); 
	    } else { 
		input->draw(X, Y, W, H, values[R][C], values[R][TYPE_COL]);
	    }

	    return;
	}

	case CONTEXT_RC_RESIZE:
	{
	    if (!visible()) return;
	    if (!input->visible()) return;
	    find_cell(CONTEXT_TABLE, row_edit, col_edit, X, Y, W, H);
	    if (X==input->x() && Y==input->y() && W==input->w() && H==input->h()) 
	        { return; }
	    input->resize(X,Y,W,H);
	    return;
	}

	default:
	    return;
    }
}



void Footer::init(void)
{
	char *buf;
	char *err = NULL;
	int n, mb;
	Document *doc = ((Document*)parent()->parent());

	sqlite3 *db = ((Document*)parent()->parent())->db;

	mb = 2048;
	buf = (char*) malloc(mb);
	snprintf(buf, 1000, "CREATE TABLE footer "
		"(key TEXT PRIMARY KEY, value TEXT); BEGIN;");
	if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
		fl_alert(_("Erreur SQL : %s."), err);
	}

	n = rows();
	while (n > 0) {
		char *v;
		int l, l2;
		
		n--;
		if (values[n][NAME_COL]) {	
			if (values[n][1]) {
				v = values[n][1];
				the_app->settings->app->get(values[n][NAME_COL], v, v);
				free(values[n][1]);
			} else {
				the_app->settings->app->get(values[n][NAME_COL], v, "");
			}
			values[n][1] = strdup(v);
			l = strlen(v);
			if (mb < l * 2 + 1024) {
				free(buf);
				mb = l * 2 + 2048;
				buf = (char*) malloc(mb);
			}
			snprintf(buf, 1000, "INSERT INTO footer (key, value) "
				"VALUES ('%s', '", values[n][NAME_COL]);
	
			l2 = strlen(buf);
			l2 += sqlite3_encode_binary((uchar*)v, l, (uchar*)buf + l2);
			snprintf(buf + l2, 1000 + l * 2 - l2, "');");
			//printf("%s '%s'\n", buf, v);
			if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
				fl_alert(_("Erreur SQL : %s."), err);
			}
		}
	}
	snprintf(buf, 1000, "COMMIT;");
	if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
		fl_alert(_("Erreur SQL : %s."), err);
	}
	free(buf);
}

static int curr_row;
static int sql_load_cb(void *data, int argc, char **argv, char **azColName){
  Footer *p = (Footer*)data;
  int l;
  if (argc < 1) return -1;
  free(p->values[curr_row][1]);
  p->values[curr_row][1] = NULL;
  if (!argv[0]) return 0;
  l = sqlite3_decode_binary((uchar*)argv[0], (uchar*)argv[0]);
  argv[0][l] = 0;
  p->values[curr_row][1] = strdup(argv[0]);
  return 0;
}

void Footer::load(void)
{
	char *err = NULL;
	sqlite3 *db = ((Document*)parent()->parent())->db;
	int n;
	char buf[1024];
	n = rows();
	while (n > 0) {
		n--;
		if (values[n][NAME_COL]) {
			curr_row = n;
			snprintf(buf, 1000, "SELECT value FROM footer "
				"WHERE key = '%s';", values[n][NAME_COL]); 
			if (sqlite3_exec(db, buf, sql_load_cb, this, &err) !=  SQLITE_OK) {
				fl_alert(_("Erreur SQL : %s."), err);
			}
		}
	}
}

void Footer::print(PrintContext *ctx)
{
	int x, y;
	PageSetup *stp = ((Document*)parent()->parent())->pagesetup;
	char *ptr;
	int ret = 0;

	x = ctx->x = 0;
	y = ctx->y;

	if (stp->changed_ != last_changed_pagesetup) {
		last_changed_pagesetup = stp->changed_;
		stp->get_font("invoice_foot_com1_font", &font1, &size1, &under1);
		stp->get_font("invoice_foot_com2_font", &font2, &size2, &under2);
	}
	
	fl_color(FL_BLACK);
	fl_font(font1, size1);
	y = y + fl_height() - fl_descent();

	ptr = get_string("foot_com1_break");
	if (ptr && atol(ptr) > 0) {
		ctx->tk->pop_clip();
		ctx->end_page();
		ctx->page_num++;
		ctx->y = y = 0;
		ctx->x = x = 0;
		ctx->start_page();
		ctx->tk->clip(0, 0, ctx->user_width, ctx->user_height);
	}

	ptr = get_string("foot_com1");
	if (ptr && ptr[0]) {
		int l = strlen(ptr);
		y = ctx->tk->draw_multi(ptr, 0, y, ctx->user_width, under1);
	}
	x = 0;
	ret = 0;

	ptr = get_string("foot_com2_break");
	if (ptr && atol(ptr) > 0) {
		ctx->tk->pop_clip();
		ctx->end_page();
		ctx->page_num++;
		ctx->y = y = 0;
		ctx->x = x = 0;
		ctx->start_page();
		ctx->tk->clip(0, 0, ctx->user_width, ctx->user_height);
	}

	fl_font(font2, size2);
	ptr = get_string("foot_com2");
	if (ptr && ptr[0]) {
		int l = strlen(ptr);
		y = ctx->tk->draw_multi(ptr, 0, y, ctx->user_width, under2);
	}

	ctx->y = y;
	
}

//
// End of "$Id: $".
//

