//
// "$Id: $"
//
// Pref dialog. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//



#include "PageSetup.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/fl_draw.H>
#include "TheApp.h"
#include "number.h"
#include "global.h"

#define NAME_COL 2
#define TYPE_COL 3


PageSetup::PageSetup(int X, int Y, int W, int H) :
	 Fl_Table(X, Y, W, H)
{
	int n, i;
	changed_ = 0;
	Fl_Group *grp = (Fl_Group*)parent();

	grp->label(_("Mise en page"));
	grp->labelfont(FL_HELVETICA_BOLD);

	input = new TableInput();
	input->callback(input_callback1, this);

	row_edit = 0;
	col_edit = 0;
	col_names[0] = _("Objet");
	col_names[1] = _("Valeur");
	col_width(0, 220);
	col_width(1, 370);
	
	callback(&event_callback1, (void*)this);
	when(FL_WHEN_NOT_CHANGED|when());
	rows(70);
	cols(2);
	col_header(1);
	col_resize(1);
	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll
	end();
	
	n = rows();
	values = (char***) malloc(sizeof(char**) * n);
	while (n > 0) {
		int m = cols() + 2;
		n--;
		values[n] = (char**) malloc(sizeof(char*) * m);
		while (m > 0) {
			m--;
			values[n][m] = NULL;
		}
	}

	i = 0;
	i++;
	values[i][0] = _("Prix indiqu\303\251s TVA inclue");
	values[i][NAME_COL] = "invoice_vat_included";
	values[i][TYPE_COL] = TBLI_CHECK;
	i++;
	i++;
	values[i][0] = _("Image ent\303\252te");
	values[i][NAME_COL] = "comp_head_pict";
	values[i][TYPE_COL] = TBLI_PICT;
	i++;
	values[i][0] = _("Largeur image ent\303\252te (mm)");
	values[i][NAME_COL] = "comp_head_pict_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
/*
	values[i][0] = _("Hauteur image ent\303\252te (mm)");
	values[i][NAME_COL] = "comp_head_pict_h";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
*/
	values[i][0] = _("Police adresse entreprise");
	values[i][NAME_COL] = "comp_address_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police lieu entreprise");
	values[i][NAME_COL] = "comp_city_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	i++;
	values[i][0] = _("Nom entreprise");
	values[i][NAME_COL] = "comp_name";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("D\303\251partement entreprise");
	values[i][NAME_COL] = "comp_dept";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;	
	values[i][0] = _("Rue entreprise");
	values[i][NAME_COL] = "comp_street";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;	
	values[i][0] = _("N\302\260 postal entreprise");
	values[i][NAME_COL] = "comp_post";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;	
	values[i][0] = _("Lieu entreprise");
	values[i][NAME_COL] = "comp_city";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Etat entreprise");
	values[i][NAME_COL] = "comp_state";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;	
	values[i][0] = _("Pays entreprise");
	values[i][NAME_COL] = "comp_country";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;	
	values[i][0] = _("T\303\251l\303\251phone entreprise");
	values[i][NAME_COL] = "comp_phone";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Mobile entreprise");
	values[i][NAME_COL] = "comp_mobile";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Fax entreprise");
	values[i][NAME_COL] = "comp_fax";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("E-mail entreprise");
	values[i][NAME_COL] = "comp_email";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Site web entreprise");
	values[i][NAME_COL] = "comp_web";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("N\302\260 TVA entreprise");
	values[i][NAME_COL] = "comp_vat";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	i++;
	values[i][0] = _("Police lieu client");
	values[i][NAME_COL] = "cust_city_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;	
	values[i][0] = _("Police adresse client");
	values[i][NAME_COL] = "cust_address_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Pos. horiz. adresse client (mm)");
	values[i][1] = strdup("120");
	values[i][NAME_COL] = "cust_address_x";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Pos. vert. adresse client (mm)");
	values[i][1] = strdup("48");
	values[i][NAME_COL] = "cust_address_y";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	i++;
	values[i][0] = _("Police lieu/date");
	values[i][NAME_COL] = "invoice_date_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Pos. horiz. lieu/date (mm)");
	values[i][1] = strdup("100");
	values[i][NAME_COL] = "invoice_date_x";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Pos. vert. lieu/date(mm)");
	values[i][1] = strdup("35");
	values[i][NAME_COL] = "invoice_date_y";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Lieu, le ...");
	values[i][1] = strdup(", le ");
	values[i][NAME_COL] = "invoice_city_date";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	i++;
	values[i][0] = _("Police No Facture");
	values[i][NAME_COL] = "invoice_no_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police titre facture");
	values[i][NAME_COL] = "invoice_title_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;		
	values[i][0] = _("Police description ent\303\252te");
	values[i][NAME_COL] = "invoice_head_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Pos. vert. d\303\251but ent\303\252te(mm)");
	values[i][1] = strdup("90");
	values[i][NAME_COL] = "invoice_head_y";
	values[i][TYPE_COL] = TBLI_INT;

	i++;
	values[i][0] = _("Concerne");
#ifdef INVOICE
	values[i][1] = strdup(_("Facture "));
#else
	values[i][1] = strdup(_("Devis estimatif "));
#endif	
	values[i][NAME_COL] = "invoice_head_about";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	i++;

	values[i][0] = _("Police ent\303\252te colonnes");
	values[i][NAME_COL] = "invoice_col_head_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police ligne normale");
	values[i][NAME_COL] = "invoice_normal_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police ligne section");
	values[i][NAME_COL] = "invoice_sect_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police ligne sous-total");
	values[i][NAME_COL] = "invoice_subtot_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police ligne total");
	values[i][NAME_COL] = "invoice_total_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	i++;
	values[i][0] = _("Largeur colonne Pos. (mm)");
	values[i][1] = strdup("15");
	values[i][NAME_COL] = "invoice_col0_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Largeur colonne No Art. (mm)");
	values[i][1] = strdup("0");
	values[i][NAME_COL] = "invoice_col1_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
/*
	values[i][0] = _("Largeur colonne Description (mm)");
	values[i][NAME_COL] = "invoice_col2_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
*/
	values[i][0] = _("Largeur colonne TVA% (mm)");
	values[i][1] = strdup("16");
	values[i][NAME_COL] = "invoice_col3_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Largeur colonne Quantit\303\251 (mm)");
	values[i][1] = strdup("16");
	values[i][NAME_COL] = "invoice_col4_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Largeur colonne Unit\303\251 (mm)");
	values[i][1] = strdup("10");
	values[i][NAME_COL] = "invoice_col5_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Largeur colonne Prix (mm)");
	values[i][1] = strdup("21");
	values[i][NAME_COL] = "invoice_col6_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Largeur colonne % (mm)");
	values[i][1] = strdup("10");
	values[i][NAME_COL] = "invoice_col7_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	values[i][0] = _("Largeur colonne Montant (mm)");
	values[i][1] = strdup("21");
	values[i][NAME_COL] = "invoice_col8_w";
	values[i][TYPE_COL] = TBLI_INT;
	i++;
	i++;
	values[i][0] = _("Lib\303\251ll\303\251 sous-total");
	values[i][1] = strdup(_("Sous total"));
	values[i][NAME_COL] = "invoice_sub_tot";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne Pos.");
	values[i][1] = strdup(_("Pos."));
	values[i][NAME_COL] = "invoice_col0_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne No Art.");
	values[i][1] = strdup(_("No Art."));
	values[i][NAME_COL] = "invoice_col1_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne Description");
	values[i][1] = strdup(_("Description"));
	values[i][NAME_COL] = "invoice_col2_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne TVA%");
	values[i][1] = strdup(_("TVA%"));
	values[i][NAME_COL] = "invoice_col3_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne Quantit\303\251");
	values[i][1] = strdup(_("Quantit\303\251"));
	values[i][NAME_COL] = "invoice_col4_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne Unit\303\251");
	values[i][1] = strdup(_("Unit\303\251"));
	values[i][NAME_COL] = "invoice_col5_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne Prix");
	values[i][1] = strdup(_("Prix"));
	values[i][NAME_COL] = "invoice_col6_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne %");
	values[i][1] = strdup(_("%"));
	values[i][NAME_COL] = "invoice_col7_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	values[i][0] = _("Ent\303\252te colonne Montant");
	values[i][1] = strdup(_("Montant"));
	values[i][NAME_COL] = "invoice_col8_t";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
	i++;

	values[i][0] = _("Police commentaires de fin 1");
	values[i][NAME_COL] = "invoice_foot_com1_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	values[i][0] = _("Police commentaires de fin 2");
	values[i][NAME_COL] = "invoice_foot_com2_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	i++;
	values[i][0] = _("Police pied de page");
	values[i][NAME_COL] = "invoice_footer_font";
	values[i][TYPE_COL] = TBLI_FONT;
	i++;
	row_height(i, 40);
	values[i][0] = _("Pied de page \n(%n = page, %p = nb pages)");
	values[i][1] = strdup(_("Page %n/%p"));
	values[i][NAME_COL] = "invoice_footer";
	values[i][TYPE_COL] = TBLI_TEXT;
	i++;
}

PageSetup::~PageSetup()
{
	int n;
	n = rows();
	while (n > 0) {
		int m = cols() + 2;
		n--;
		while (m > 0) {
			m--;
			if (m == 1) {
				// free only the value entered by the user
				free(values[n][m]);
			}
		}
		free(values[n]);
	}
	free(values);
}

void PageSetup::add_rows()
{
	int n = rows();

	rows(rows() + 8);
	
	values = (char***) realloc(values, 
		sizeof(char**) * rows());
	while (n < rows()) {
		int m = cols() + 2;
		values[n] = (char**) malloc(sizeof(char*) * m);
		while (m > 0) {
			m--;
			values[n][m] = NULL;
		}
		n++;
	}
}

void PageSetup::event_callback1(Fl_Widget *w, void *d)
{
	((PageSetup*)d)->event_callback();
}

void PageSetup::input_callback1(Fl_Widget *w, void *d)
{
	((PageSetup*)d)->input_callback();
}

void PageSetup::input_callback()
{
	int w, h;
	w = col_width(col_edit) - 6;
	h = row_height(row_edit) - 6;
	free(values[row_edit][col_edit]);
	
	values[row_edit][col_edit] = NULL;

	if (input->value() && input->value()[0])
			values[row_edit][col_edit] = strdup(input->value());

	input->hide();
	take_focus();

	set(row_edit);
}

void PageSetup::get_value(char* name, char** value, char** type)
{
	int n = rows();

	*type = NULL;
	*value = NULL;
	while (n > 0) {
		n--;
		if (values[n][NAME_COL] && !strcmp(name, values[n][NAME_COL])) {
			*type = values[n][TYPE_COL];
			*value = values[n][1]; 
			return;
		}
	}
}

void PageSetup::get_font(char *name, Fl_Font *fnt, int *size, int *underline)
{
	char *value;
	char *type;
	get_value(name, &value, &type);
	input->get_font(value, type, fnt, size, underline);
}

int PageSetup::get_int(char *name)
{
	char *value;
	char *type;
	get_value(name, &value, &type);
	
	if (!value) return 0;
	return atol(value);
}

char *PageSetup::get_string(char *name)
{
	char *value;
	char *type;
	get_value(name, &value, &type);
	return value;
}

static int sql_no_cb(void *data, int argc, char **argv, char **azColName){
  PageSetup *p = (PageSetup*)data;
  return 0;
}

void PageSetup::set(int row)
{
	char *buf;
	int l;
	char *err = NULL;
		
	sqlite3 *db = ((Document*)parent()->parent()->parent())->db;

	if (!values[row][NAME_COL]) return;

	((Document*)parent()->parent()->parent())->changed_++;
	changed_++;

	
	if (!values[row][1]) {
		l = 1024;
		buf = (char*) malloc(1024);
		snprintf(buf, 1000, 
			"UPDATE page_setup SET value = '' WHERE  "
			 "key = '%s';", values[row][NAME_COL]);
		the_app->settings->app->set(values[row][NAME_COL], "");
	} else {
		int l2;
		l = strlen(values[row][1]);
		buf = (char*) malloc(l * 2 + 1024);
		snprintf(buf, 1000, 
			"UPDATE page_setup SET value = '");
		l2 = strlen(buf);
		l2 += sqlite3_encode_binary((uchar*)values[row][1], l, (uchar*)buf + l2);
		snprintf(buf + l2, 1000 + l * 2 - l2, "' WHERE "
			"key = '%s';", values[row][NAME_COL]);
		the_app->settings->app->set(values[row][NAME_COL], values[row][1]);	
	}

	if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
		fl_alert(_("Erreur SQL : %s."), err);
	}
	free(buf);
}


void PageSetup::event_callback()
{
    int R = callback_row(),
        C = callback_col();
    TableContext context = callback_context();

    switch ( context )
    {
	case CONTEXT_CELL:
	{
            switch (Fl::event()) 
	    { 
		case FL_PUSH:
		    row_edit = R;
		    col_edit = C;
		    if (!Fl::event_clicks()) 
		    {
			if (input->visible()) input->run_callback();
			input->hide();
			return;
		    }
		    Fl::event_clicks(0);
		    //FALLTHROUGH

		case FL_KEYBOARD:
		    row_edit = R;
		    col_edit = C;

		 
		    if (Fl::event() == FL_KEYBOARD && Fl::e_length == 0)
		        { return; }   
		    if (input->visible()) {
				input->run_callback();
		    }
		    
		    //if (R > rows() - 3) add_rows();
		    if (C < 1) {
			input->hide();
			return;
		    }	
		    set_selection(R, C, R, C);

		    int XX,YY,WW,HH;
		    find_cell(CONTEXT_CELL, R, C, XX, YY, WW, HH);
		    input->resize(XX,YY,WW,HH);
		    input->value(values[R][C]);
		    input->show(values[R][TYPE_COL]);
		    if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') 
			{ input->handle(Fl::event()); }
		    return;
	    }
	    return;
	}

	case CONTEXT_ROW_HEADER:
	case CONTEXT_COL_HEADER:
	    if (input->visible()) input->run_callback();
            input->hide();
	    return;

	case CONTEXT_TABLE:
	    if (R < 0 && C < 0)
	    {
	        if (input->visible()) input->run_callback();
                input->hide();
	    }
	    return;
	
	default:
	    return;
    }
}

void PageSetup::draw_cell(TableContext context,
	int R, int C, int X, int Y, int W, int H)
{

    switch ( context )
    {
	case CONTEXT_STARTPAGE:
	    // Get kb nav + mouse 'selection region' for use below
	    get_selection(s_top, s_left, s_bottom, s_right);
	    s_top = s_bottom;
	    s_left = s_right; 	
	    break;

	case CONTEXT_COL_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, col_header_color());
		fl_color(FL_BLACK);
		fl_draw(col_names[C],  X, Y, W, H, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_ROW_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{
	    if (R == row_edit && C == col_edit && input->visible())
	        { return; }

	    if (R >= s_top && R <= s_bottom && C >= s_left && C <= s_right) { 
		input->draw_selected(X, Y, W, H, values[R][C], values[R][TYPE_COL]); 
	    } else { 
		input->draw(X, Y, W, H, values[R][C], values[R][TYPE_COL]);
	    }

	    return;
	}

	case CONTEXT_RC_RESIZE:
	{
	    if (!visible()) return;
	    if (!input->visible()) return;
	    find_cell(CONTEXT_TABLE, row_edit, col_edit, X, Y, W, H);
	    if (X==input->x() && Y==input->y() && W==input->w() && H==input->h()) 
	        { return; }
	    input->resize(X,Y,W,H);
	    return;
	}

	default:
	    return;
    }
}



void PageSetup::init(void)
{
	char *buf;
	char *err = NULL;
	int n, mb;

	sqlite3 *db = ((Document*)parent()->parent()->parent())->db;

	mb = 2048;
	buf = (char*) malloc(mb);
	snprintf(buf, 1000, "CREATE TABLE page_setup "
		"(key TEXT PRIMARY KEY, value TEXT); BEGIN;");
	if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
		fl_alert(_("Erreur SQL : %s."), err);
	}

	n = rows();
	while (n > 0) {
		char *v;
		int l, l2;
		
		n--;
		if (values[n][NAME_COL]) {	
			if (values[n][1]) {
				v = values[n][1];
				the_app->settings->app->get(values[n][NAME_COL], v, v);
				free(values[n][1]);
			} else {
				the_app->settings->app->get(values[n][NAME_COL], v, "");
			}
			values[n][1] = strdup(v);
			l = strlen(v);
			if (mb < l * 2 + 1024) {
				free(buf);
				mb = l * 2 + 2048;
				buf = (char*) malloc(mb);
			}
			snprintf(buf, 1000, "INSERT INTO page_setup (key, value) "
				"VALUES ('%s', '", values[n][NAME_COL]);
	
			l2 = strlen(buf);
			l2 += sqlite3_encode_binary((uchar*)v, l, (uchar*)buf + l2);
			snprintf(buf + l2, 1000 + l * 2 - l2, "');");
			//printf("%s '%s'\n", buf, v);
			if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
				fl_alert(_("Erreur SQL : %s."), err);
			}
		}
	}
	snprintf(buf, 1000, "COMMIT;");
	if (sqlite3_exec(db, buf, sql_no_cb, this, &err) !=  SQLITE_OK) {
		fl_alert(_("Erreur SQL : %s."), err);
	}
	free(buf);
}

static int curr_row;
static int sql_load_cb(void *data, int argc, char **argv, char **azColName){
  PageSetup *p = (PageSetup*)data;
  int l;
  if (argc < 1) return -1;
  free(p->values[curr_row][1]);
  p->values[curr_row][1] = NULL;
  if (!argv[0]) return 0;
  l = sqlite3_decode_binary((uchar*)argv[0], (uchar*)argv[0]);
  argv[0][l] = 0;
  p->values[curr_row][1] = strdup(argv[0]);
  return 0;
}

void PageSetup::load(void)
{
	char *err = NULL;
	sqlite3 *db = ((Document*)parent()->parent()->parent())->db;
	int n;
	char buf[1024];
	n = rows();
	while (n > 0) {
		n--;
		if (values[n][NAME_COL]) {
			curr_row = n;
			snprintf(buf, 1000, "SELECT value FROM page_setup "
				"WHERE key = '%s';", values[n][NAME_COL]); 
			if (sqlite3_exec(db, buf, sql_load_cb, this, &err) !=  SQLITE_OK) {
				fl_alert(_("Erreur SQL : %s."), err);
			}
		}
	}
}

//
// End of "$Id: $".
//


