//
// "$Id: $"
//
// Position. 
//
//                   Copyright (c) 2004-2005, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "global.h"
#include "Position.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/fl_draw.H>
#include "TheApp.h"
#include "number.h"

#define QUANTITY_COL 5
#define PRICE_COL 7
#define TOTAL_COL 9
#define PC_COL 8
#define TVA_COL 4
#define TYPE_COL 0
#define POS_COL 1
#define ART_COL 2
#define DESC_COL 3
#define UNIT_COL 6
#define NB_COL 10
#define NORMAL_CL 'a'
#define BEG_SEC_CL 'b'
#define BREAK_CL 'c'
#define END_SEC_CL 'e'
#define END_PAGE_CL 'd'
#define COMMENT_CL 'f'

static Fl_Menu_Item popup[] = {
	{"Normal", 0, 0, 0, 0},
	{"D\303\251but section", 0, 0, 0, 0},
	{"Fin section", 0, 0, 0, 0},
	{"Saut de page", 0, 0, 0, 0},
	{"Commentaire", 0, 0, 0, 0},
	{0}};


Position::Position(int X, int Y, int W, int H) :
	 Fl_Table(X, Y, W, H)
{
	int n;
	char buf[1024];

	draw_id = 0;

	label(_("Positions"));
	labelfont(FL_HELVETICA_BOLD);
	input = new PositionInput(X + 50, Y + 50, 10, 10);
	input->hide();
	input->callback(input_callback1, this);
	input->when(0);

	menu = new Fl_Menu_Button(X + 50, Y + 50, 10, 10);
	menu->menu(popup);
	menu->callback(menu_callback1, this);
	menu->hide();

	sub_tot = NULL;

	row_edit = 0;
	col_edit = 0;
	col_names[TYPE_COL] = _("Type");
	col_names[POS_COL] = _("Pos.");
	col_names[ART_COL] = _("No Art.");
	col_names[DESC_COL] = _("Description");
	col_names[TVA_COL] = _("TVA%");
	col_names[QUANTITY_COL] = _("Quantit\303\251");
	col_names[UNIT_COL] = _("Unit\303\251");
	col_names[PRICE_COL] = _("Prix");
	col_names[PC_COL] = _("%");
	col_names[TOTAL_COL] = _("Montant");

	Fl_Preferences *app = the_app->settings->app;

	for (n = 0; n < NB_COL; n++) {
		int v;	
		switch (n) {
		case DESC_COL:
			v = 300;
			break;
		case TVA_COL:
		case UNIT_COL:
		case PC_COL:
			v = 45;
			break;
		case TYPE_COL:
			v = 40;
			break;
		default:
			v = 80;
			break;
		}
		snprintf(buf, 1000, "posi_table_col_width[%d]", n);
		app->get(buf,v, v); 
		col_width(n, v);
	}	
	
	callback(&event_callback1, (void*)this);
	when(FL_WHEN_NOT_CHANGED|when());
	rows(40);
	cols(10);
	col_header(1);
	col_resize(1);
	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll
	end();
	
	n = rows();
	values = (char***) malloc(sizeof(char**) * n);
	while (n > 0) {
		int m = cols();
		n--;
		values[n] = (char**) malloc(sizeof(char*) * m);

		
		while (m > 0) {
			m--;
			values[n][m] = NULL;
		}
		// row type
		values[n][TYPE_COL] = (char*)malloc(3);
		values[n][TYPE_COL][0] = NORMAL_CL; // type
		values[n][TYPE_COL][1] = 0; // draw_id;
		values[n][TYPE_COL][2] = 0; 
	}
	
}

Position::~Position()
{
	char buf[1024];
	int n;
	Fl_Preferences *app = the_app->settings->app;
	for (n = 0; n < 10; n++) {
		snprintf(buf, 1000, "posi_table_col_width[%d]", n);
		app->set(buf, col_width(n)); 
	}

	n = rows();
	while (n > 0) {
		int m = cols();
		n--;
		while (m > 0) {
			m--;
			free(values[n][m]);
		}
		free(values[n]);
	}
	free(values);
}

void Position::add_rows()
{
	make_room(rows(), 8);
}

// start is the index of the first newly created row
void Position::make_room(int start, int nb)
{
	int n; 
	int r;
	int i;

	printf("make room %d %d\n", start, nb);

	if (start < 0) start = 0;
	if (start > rows()) start = rows();
	
	r = rows();
	rows(rows() + nb);
	
	values = (char***) realloc(values, 
		sizeof(char**) * rows());

	for (n = r ; n < rows(); n++) {
		int m = cols();

		values[n] = (char**) malloc(sizeof(char*) * m);

		while (m > 0) {
			m--;
			values[n][m] = NULL;
		}
		// row type
		values[n][TYPE_COL] = (char*)malloc(3);
		values[n][TYPE_COL][0] = NORMAL_CL; // type
		values[n][TYPE_COL][1] = 0; // draw_id;
		values[n][TYPE_COL][2] = 0;
	}

	for (n = rows() - 1, i = r - 1; i >= start; i--, n--) {
		char **tmp = values[n];
		values[n] = values[i];
		values[i] = tmp;
	} 
}

void Position::event_callback1(Fl_Widget *w, void *d)
{
	((Position*)d)->event_callback();
}

void Position::input_callback1(Fl_Widget *w, void *d)
{
	((Position*)d)->input_callback();
}

void Position::menu_callback1(Fl_Widget *w, void *d)
{
	((Position*)d)->menu_callback();
}

void Position::menu_callback()
{
	char m;

	switch (menu->value()) {
	case 1:
		m = BEG_SEC_CL; break;
	case 2:
		m = END_SEC_CL; break;
	case 3:
		m = BREAK_CL; break; 
	case 4:
		m = COMMENT_CL; break;
	default:
		m = NORMAL_CL;
	}
	if (values[row_edit][TYPE_COL][0] != END_PAGE_CL) {
		values[row_edit][TYPE_COL][0] = m;
		redraw();
	}
	if (m == END_SEC_CL && !values[row_edit][DESC_COL]) {
		values[row_edit][DESC_COL] = strdup(_("Sous-total"));
	}
	recompute();
}

void Position::input_callback()
{
	int w, h;
	i64 q, old;
	w = col_width(col_edit) - 6;
	h = row_height(row_edit) - 6;

	free(values[row_edit][col_edit]);
	values[row_edit][col_edit] = NULL;

	switch(col_edit) {
	case PC_COL:
	case PRICE_COL:
	case TVA_COL:
		changed_++;
		((Document*)parent()->parent())->changed_++;

		if (input->value() && input->value()[0])
			values[row_edit][col_edit] = 
				strdup(num_string(string_num(input->value())));	
		break;
	default:
		if (input->value() && input->value()[0])
			values[row_edit][col_edit] = strdup(input->value());
	}
	input->hide();
	take_focus();
	fl_font(FL_HELVETICA, 14);
	fl_measure(values[row_edit][col_edit], w, h);
	if (h > row_height(row_edit) - 6) {
		row_height(row_edit, h + 6);
	}


	if (values[row_edit][TYPE_COL][0] != NORMAL_CL) {
		set(row_edit);
		return;
	}

	old = string_num(values[row_edit][TOTAL_COL]);
	q = string_num(values[row_edit][QUANTITY_COL]) *
		string_num(values[row_edit][PRICE_COL]) / 1000;
	q -= q * string_num(values[row_edit][PC_COL]) / 100000;
	q = arrondi(q);


	if (q != old) {
		int i;
		free(values[row_edit][TOTAL_COL]);
		if (values[row_edit][QUANTITY_COL] && 
			values[row_edit][QUANTITY_COL][0])
		{
			values[row_edit][TOTAL_COL] = strdup(num_string(q));
		} else {
			values[row_edit][TOTAL_COL] = NULL;
		}
		for (i = row_edit + 1; i < rows(); i++) {
			if (values[i][TYPE_COL][0] == END_SEC_CL) {
				i64 t;
				t = string_num(values[i][TOTAL_COL]);
				t += q - old;
				free(values[i][TOTAL_COL]);
				values[i][TOTAL_COL] = strdup(num_string(t));
				set(i);
				break; 
			}
		}
	}
	set(row_edit);		
}

void Position::recompute()
{
	i64 old, q, total, sub;
	int row_edit;
	
	total = 0;
	sub = 0;
	for (row_edit = 0; row_edit < rows(); row_edit++) {
		old = string_num(values[row_edit][TOTAL_COL]);
		q = 0;
		if (values[row_edit][TYPE_COL][0] == NORMAL_CL) {
			q = string_num(values[row_edit][QUANTITY_COL]) *
			string_num(values[row_edit][PRICE_COL]) / 1000;
			q -= q * string_num(values[row_edit][PC_COL]) / 100000;
			q = arrondi(q);
			sub += q;
			total += q;
		} else if (values[row_edit][TYPE_COL][0] == BEG_SEC_CL) {
			sub = 0;
		} else if (values[row_edit][TYPE_COL][0] == END_SEC_CL) {
			q = sub;
			sub = 0;
		}

		if (q != old) {
			int i;
			free(values[row_edit][TOTAL_COL]);
			if ((values[row_edit][QUANTITY_COL] && 
				values[row_edit][QUANTITY_COL][0]) ||
				values[row_edit][TYPE_COL][0] == END_SEC_CL)
			{
				values[row_edit][TOTAL_COL] = strdup(num_string(q));
			} else {
				values[row_edit][TOTAL_COL] = NULL;
			}
		}
	}
	redraw();
}


void Position::event_callback()
{
    int R = callback_row(),
        C = callback_col();
    TableContext context = callback_context();


    switch ( context )
    {
	case CONTEXT_CELL:
	{
            switch (Fl::event()) 
	    { 	
		case FL_PASTE:
			paste();
			return;	
		case FL_PUSH:
		    if (!Fl::event_clicks()) 
		    {
			if (input->visible()) input->do_callback();
			input->hide();
			return;
		    }
		    Fl::event_clicks(0);
		    //FALLTHROUGH

		case FL_KEYBOARD:
		    if (Fl::event() == FL_KEYBOARD && Fl::e_length == 0)
		        { return; }   
		    if (input->visible()) {
				input->do_callback();
		    } else if (Fl::event_state() & FL_CTRL) {
			switch (Fl::event_text()[0]) {
			case ('C'^0x40):
				select();
				return;
			case ('V'^0x40):
				Fl::paste(*this, 1);
				return;
			case ('X'^0x40):
				cut();
				return;
			}
		    }
		    if (C == cols()) return;
		    if (R > rows() - 3) add_rows();
		    int XX,YY,WW,HH;
		    int type = (unsigned char)values[R][TYPE_COL][0];
		    
		    row_edit = R;
		    col_edit = C;
		    find_cell(CONTEXT_CELL, R, C, XX, YY, WW, HH);
		   
		    set_selection(R, C, R, C);
		    if (C == TYPE_COL) {
			menu->resize(XX,YY,WW,HH);
			menu->popup();
		    } else if (type != BREAK_CL && type != END_PAGE_CL) {
			input->resize(XX,YY,WW,HH);
			input->value(values[R][C]);
			input->show();
			input->take_focus();
			if (Fl::event() == FL_KEYBOARD && Fl::e_text[0] != '\r') 
				{ input->handle(Fl::event()); }
		    }
		    return;
	    }
	    return;
	}

	case CONTEXT_ROW_HEADER:
	case CONTEXT_COL_HEADER:
	    if (input->visible()) input->do_callback();
            input->hide();
	    return;

	case CONTEXT_TABLE:
	    if (R < 0 && C < 0)
	    {
	        if (input->visible()) input->do_callback();
                input->hide();
	    }
	    return;
	
	default:
	    return;
    }
}

void Position::draw_cell(TableContext context,
	int R, int C, int X, int Y, int W, int H)
{

    switch ( context )
    {
	case CONTEXT_STARTPAGE:
	    // Get kb nav + mouse 'selection region' for use below
	    get_selection(s_top, s_left, s_bottom, s_right);
	    draw_id++;	
	    break;

	case CONTEXT_COL_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, col_header_color());
		fl_color(FL_BLACK);
		fl_draw(col_names[C],  X, Y, W, H, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_ROW_HEADER:
	    fl_font(FL_HELVETICA | FL_BOLD, 14);
	    fl_push_clip(X, Y, W, H);
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{
	    int type = (unsigned char)values[R][TYPE_COL][0];

	    // don't draw twice.
	    //if (values[R][TYPE_COL][1] == draw_id) {
		//return;
	    //}
	    values[R][TYPE_COL][1] = draw_id;
		
	    
	    if (R == row_edit && C == col_edit && input->visible())
	        { return; }

	    // BACKGROUND
	    fl_push_clip(X, Y, W, H);

	    // Keyboard nav and mouse selection highlighting
	    if (R >= s_top && R <= s_bottom && C >= s_left && C <= s_right)
                { fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_YELLOW); }
            else
	        { fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_WHITE); }

	    fl_pop_clip();

	    // TEXT
	    fl_push_clip(X+3, Y+3, W-6, H-6);
	    {
		int a;
		switch (type) {
		case BREAK_CL:
		
		default:
			fl_color(FL_BLACK);
		}
		a = FL_ALIGN_WRAP|FL_ALIGN_RIGHT|FL_ALIGN_TOP;
		switch(C) {
		case POS_COL:
		case ART_COL:
		case DESC_COL:
		case UNIT_COL:
			a = FL_ALIGN_WRAP|FL_ALIGN_LEFT|FL_ALIGN_TOP;	
		}
		if (C >0) {
			fl_font(FL_HELVETICA_BOLD, 14);
			switch (type) {
			case BEG_SEC_CL:
				// section title
				if (values[R][C]) fl_draw(values[R][C], X+3, Y+3, W-6, H-6, (Fl_Align)a);
				break;	
			case BREAK_CL:
				// page break
				fl_color(FL_RED);
				if (C == DESC_COL) fl_draw(_("Saut de page"), X+3, Y+3, W-6, H-6, (Fl_Align)a);
				break;
			case END_PAGE_CL:
				// end of page
				fl_color(FL_BLUE);
				if (C == DESC_COL) fl_draw(_("Fin de page"), X+3, Y+3, W-6, H-6, (Fl_Align)a);
				break;
			case END_SEC_CL:
				// sub total
				if (values[R][C]) fl_draw(values[R][C], X+3, Y+3, W-6, H-6, (Fl_Align)a);
				break;
			case COMMENT_CL:
				fl_font(FL_HELVETICA_ITALIC, 14);
				if (values[R][C]) fl_draw(values[R][C], X+3, Y+3, W-6, H-6, (Fl_Align)a);
				break;
			default:
				fl_font(FL_HELVETICA, 14);
				fl_draw(values[R][C], X+3, Y+3, W-6, H-6, (Fl_Align)a);
			}
		} else if (C == 0) {
			char *ptr;
			switch (type) {
			default:
			case NORMAL_CL:
				ptr = _("Norm"); break;
			case BEG_SEC_CL:
				ptr = _("DS"); break;
			case BREAK_CL:
				ptr = _("SP"); break;
			case END_SEC_CL:
				ptr = _("FS"); break;
			case END_PAGE_CL:
				ptr = _("Page"); break;
			case COMMENT_CL:
				ptr = _("Com."); break;
			}
			fl_font(FL_HELVETICA, 14);
			fl_draw(ptr, X+3, Y+3, W-6, H-6, (Fl_Align)a);
		}
	    }
	    fl_pop_clip();

	    return;
	}

	case CONTEXT_RC_RESIZE:
	{
	    if (!visible()) return;
	    if (!input->visible()) return;
	    find_cell(CONTEXT_TABLE, row_edit, col_edit, X, Y, W, H);
	    if (X==input->x() && Y==input->y() && W==input->w() && H==input->h()) 
	        { return; }
	    input->resize(X,Y,W,H);
	    return;
	}

	default:
	    return;
    }
}

void Position::insert_line_before(int line, char type)
{
	if (!sub_tot) {
		PageSetup *stp = ((Document*)parent()->parent())->pagesetup;
		sub_tot = stp->get_string("invoice_sub_tot");
	}
	if (!sub_tot) sub_tot = "";

	make_room(line, 1);

	if (type == END_SEC_CL) {
		free(values[line][ART_COL]);
		values[line][ART_COL] = strdup(sub_tot);	
	}

	values[line][TYPE_COL][0] = type;
	values[line][TYPE_COL][1] = draw_id;	
}



void Position::begin_transaction(void)
{
	sqlite3 *db;
	int err;
	sqlite3_stmt *pStmt = NULL;
	char *zSql = NULL;
	const char *zTail = NULL;

	db = ((Document*)parent()->parent())->db;

	zSql = "BEGIN TRANSACTION;";
	
	err = sqlite3_prepare(db, zSql, -1, &pStmt, &zTail); 
	if (err == SQLITE_OK) {
		//sqlite3_bind_text(pStmt, 1, buffer, -1, SQLITE_STATIC);
		do {
			err = sqlite3_step(pStmt);
			if (err == SQLITE_ROW) {
				const unsigned char *col0; 
				col0 = sqlite3_column_text(pStmt, 0);
			}
		} while (err == SQLITE_ROW); 
	}

	if (err != SQLITE_DONE && err != SQLITE_OK) {
		fl_alert(_("%s %d Error #%d : %s."), __FILE__, __LINE__, err, sqlite3_errmsg(db));
	}
	sqlite3_finalize(pStmt);

}

void Position::end_transaction(void)
{
	sqlite3 *db;
	int err;
	sqlite3_stmt *pStmt = NULL;
	char *zSql = NULL;
	const char *zTail = NULL;

	db = ((Document*)parent()->parent())->db;

	zSql = "END TRANSACTION;";
	
	err = sqlite3_prepare(db, zSql, -1, &pStmt, &zTail); 
	if (err == SQLITE_OK) {
		//sqlite3_bind_text(pStmt, 1, buffer, -1, SQLITE_STATIC);
		do {
			err = sqlite3_step(pStmt);
			if (err == SQLITE_ROW) {
				const unsigned char *col0; 
				col0 = sqlite3_column_text(pStmt, 0);
			}
		} while (err == SQLITE_ROW); 
	}

	if (err != SQLITE_DONE && err != SQLITE_OK) {
		fl_alert(_("%s %d Error #%d : %s."), __FILE__, __LINE__, err, sqlite3_errmsg(db));
	}
	sqlite3_finalize(pStmt);

}

void Position::init(void)
{
	sqlite3 *db;
	int err;
	sqlite3_stmt *pStmt = NULL;
	char *zSql = NULL;
	const char *zTail = NULL;

	db = ((Document*)parent()->parent())->db;

	zSql = "CREATE TABLE position ("
		"line TEXT PRIMARY KEY, "
		"type TEXT, "
		"pos TEXT, "
		"art TEXT, "
		"desc TEXT, "
		"tva TEXT, "
		"quantity TEXT, "
		"unit TEXT, "
		"price TEXT, "
		"pc TEXT, "
		"total TEXT"
		");";
	err = sqlite3_prepare(db, zSql, -1, &pStmt, &zTail); 
	if (err == SQLITE_OK) {
		//sqlite3_bind_text(pStmt, 1, buffer, -1, SQLITE_STATIC);
		do {
			err = sqlite3_step(pStmt);
			if (err == SQLITE_ROW) {
				const unsigned char *col0; 
				col0 = sqlite3_column_text(pStmt, 0);
			}
		} while (err == SQLITE_ROW); 
	}

	if (err != SQLITE_DONE && err != SQLITE_OK) {
		fl_alert(_("%s %d Error #%d : %s."), __FILE__, __LINE__, err, sqlite3_errmsg(db));
	}
	sqlite3_finalize(pStmt);

}

void Position::select()
{
	char *buf;
	int mlen = 1024;
	int len = 0;
	int line;
	int col;

	buf = (char*) malloc(mlen);

	get_selection(s_top, s_left, s_bottom, s_right);
	printf("Select %d %d %d %d\n", s_top, s_left, s_bottom, s_right);
	s_left = 0;
	s_right = NB_COL - 1;

	for (line = s_top; line <= s_bottom; line++) {
		for (col = s_left; col <= s_right; col++) {
			char *ptr = values[line][col];
			int l = 0;
			int q = 0;
			if (!ptr || !ptr[0]) {
				if (mlen - len < 10) {
					mlen *= 2;
					buf = (char*) realloc(buf, mlen);
				}
				if (col < s_right) buf[len++] = ',';
				continue;
			}
			while (ptr[l]) {
				if (ptr[l] == '"') q++;
				l++;
			}
			if (mlen - len - 10 < l + q) {
				mlen *= 2;
				buf = (char*) realloc(buf, mlen);
			}
			l = 0;
			buf[len++] = '"';
			while (ptr[l]) {
				if (ptr[l] == '"') {
					buf[len++] = '"';
				}
				buf[len++] = ptr[l++];
			}
			buf[len++] = '"';
			if (col < s_right) buf[len++] = ',';
		}
		buf[len++] = '\n';
	}

	Fl::copy(buf, len, 1);
	free(buf);
}

static char *getstr(const char *start, const char *end)
{
	char *buf;
	int i;
	buf = (char*) malloc(end - start + 3);
	while (isspace((uchar)*start) && start < end) start++;
	
	i = 0;
	if (*start == '"') {
		start++;
		while (end > start && *end != '"') end--;
		while (start < end) {
			buf[i++] = *start;
			if (*start == '"') start++;
			start++;
		}	
	} else {
		while (start < end) {
			buf[i++] = *start;
			start++;
		}	
	}
	buf[i] = 0;
	return buf;
}

void Position::paste()
{
	int nbl = 0;
	int len = Fl::event_length();
	const char *ptr = Fl::event_text();
	int q = 0;
	int nbc = 0;
	int l = 0;
	const char *start;
	const char *end;
	int i;

	get_selection(s_top, s_left, s_bottom, s_right);
	printf("Paste %d %d %d %d %s\n", s_top, s_left, s_bottom, s_right, ptr);
	
	if (s_top != s_bottom) {
		cut(1);
	}

	s_left = 0;
	s_right = NB_COL - 1;

	while (l < len) {	
		if (q == 2) {
			if (ptr[l] == '"') q = 3;
			else q = 0;
		}
		if (!q && ptr[l] == ',') nbc++;
		if (q == 1 && ptr[l] == '"') q = 2;
		if (!q && ptr[l] == '"') q = 1;
	
		if (!q && ptr[l] == '\n' && l < len - 1) {
			nbl++;
			nbc = 0;
		}
		if (q == 3) q = 1; // was " ""
		l++;
	}
	nbl++;
	nbc++;
	make_room(s_top, nbl);

	l = 0;
	nbl = 0;
	nbc = 0;
	start = ptr;
	end = ptr;
	while (l < len) {	
		if (q == 2) {
			if (ptr[l] == '"') q = 3;
			else q = 0;
		}
		if (!q && ptr[l] == ',') {
			char *tmp;
			tmp = getstr(start, end);
			if (s_left + nbc == TYPE_COL) {
				values[s_top + nbl][TYPE_COL][0] = tmp[0];
			} else {
				if (s_left + nbc < cols()) {
					values[s_top + nbl][s_left + nbc] = tmp;
				}
			}
			start = end + 1;
			nbc++;
		}
		if (q == 1 && ptr[l] == '"') q = 2;
		if (!q && ptr[l] == '"') q = 1;
	
		if (!q && ptr[l] == '\n' && l < len - 1) {
			char *tmp;
			tmp = getstr(start, end);
			if (s_left + nbc == TYPE_COL) {
				values[s_top + nbl][TYPE_COL][0] = tmp[0];
			} else {
				if (s_left + nbc < cols()) {
					values[s_top + nbl][s_left + nbc] = tmp;
				}
			}
			start = end + 1;
			nbl++;
			nbc = 0;
		}
		if (q == 3) q = 1; // was " ""
		l++;
		end++;
	}
	{
			char *tmp;
			tmp = getstr(start, end);
			if (s_left + nbc == TYPE_COL) {
				values[s_top + nbl][TYPE_COL][0] = tmp[0];
			} else {
				if (s_left + nbc < cols()) {
					values[s_top + nbl][s_left + nbc] = tmp;
				}
			}
	}
	changed_++;
	((Document*)parent()->parent())->changed_++;

	redraw();
	Fl::flush();

	recompute();

	begin_transaction();
	for (i = rows() - 1; i >= s_top; i--) {
		set(i);
	}
	end_transaction();
	
}

void Position::cut(int paste)
{
	int i, n;
	get_selection(s_top, s_left, s_bottom, s_right);
	printf("Cut\n");

	if (!paste) select();

	s_left = 0;
	s_right = NB_COL - 1;

	for (i = s_top; i <= s_bottom; i++) {
		int m = cols();
		while (m > 0) {
			m--;
			free(values[i][m]);
		}
		free(values[i]);
	}
	for (i = s_top, n = s_bottom + 1; n < rows(); i++, n++) {
		values[i] = values[n];
	}
	for (i = s_bottom - s_top + 1; i > 0; i--) {
		int m = cols();
		int n = rows() - i;
		values[n] = (char**) malloc(sizeof(char*) * m);
		
		while (m > 0) {
			m--;
			values[n][m] = NULL;
		}
		// row type
		values[n][TYPE_COL] = (char*)malloc(3);
		values[n][TYPE_COL][0] = NORMAL_CL; // type
		values[n][TYPE_COL][1] = 0; // draw_id;
		values[n][TYPE_COL][2] = 0; 
	}

	if (paste) return;

	changed_++;
	((Document*)parent()->parent())->changed_++;

	redraw();
	Fl::flush();

	recompute();

	begin_transaction();
	for (i = rows() - 1; i >= s_top; i--) {
		set(i);
	} 
	end_transaction();
}

void Position::set(int line)
{
	sqlite3 *db;
	char buffer[1024];
	int err;
	sqlite3_stmt *pStmt = NULL;
	char *zSql = NULL;
	const char *zTail = NULL;
	char update = 0;
	char buf[128];
	char buf1[3];

	db = ((Document*)parent()->parent())->db;

	//printf("SET : line %d\n", line);

	// search if we must UPDATE or INSERT
	zSql = buffer;
	snprintf(buffer, 1000, "SELECT (line) FROM position WHERE line = '%08d' ;", line);
	
	err = sqlite3_prepare(db, zSql, -1, &pStmt, &zTail); 
	if (err == SQLITE_OK) {
		//sqlite3_bind_text(pStmt, 1, buffer, -1, SQLITE_STATIC);
		do {
			err = sqlite3_step(pStmt);
			if (err == SQLITE_ROW) {
				update = 1;
			}
		} while (err == SQLITE_ROW); 
	}

	if (err != SQLITE_DONE && err != SQLITE_OK) {
		fl_alert(_("%s %d Error #%d : %s."), __FILE__, __LINE__, err, sqlite3_errmsg(db));
	}
	sqlite3_finalize(pStmt);

	if (update) {
		snprintf(buffer, 1000, "UPDATE position SET "
			" line = ? , "
			"type = ? , "
			"pos = ? , "
			"art = ? , "
			"desc = ? , "
			"tva = ? , "
			"quantity = ? , "
			"unit = ? , "
			"price = ? , "
			"pc = ? , "
			"total = ? "
			" WHERE line = '%08d' ;", line);	
	} else {
		snprintf(buffer, 1000, "INSERT INTO position "
			"(line, "
			"type, "
			"pos, "
			"art, "
			"desc, "
			"tva, "
			"quantity, "
			"unit, "
			"price, "
			"pc, "
			"total "
			") VALUES (?,?,?,?,?,?,?,?,?,?,?);");	
	}
	
	err = sqlite3_prepare(db, zSql, -1, &pStmt, &zTail); 
	if (err == SQLITE_OK) {
		int i;
		snprintf(buf, 100, "%08d", line);
		for (i = 0; i <= NB_COL; i++) {
			const char *p = buf;
			
			switch (i) {
			case 0:
				p = buf;
				break;
			case 1:
				buf1[0] = values[line][TYPE_COL][0];
				buf1[2] = 0;
				p = buf1;
				break;
			case 2:
				p = values[line][POS_COL]; break;
			case 3:
				p = values[line][ART_COL]; break;
			case 4:
				p = values[line][DESC_COL]; break;
			case 5:
				p = values[line][TVA_COL]; break;
			case 6:
				p = values[line][QUANTITY_COL]; break;
			case 7:
				p = values[line][UNIT_COL]; break;
			case 8:
				p = values[line][PRICE_COL]; break;
			case 9:
				p = values[line][PC_COL]; break;
			case 10:
				p = values[line][TOTAL_COL]; break;
			}
			sqlite3_bind_text(pStmt, i + 1, p, -1, SQLITE_STATIC);
		}
	}
	err = sqlite3_step(pStmt);

	if (err != SQLITE_DONE && err != SQLITE_OK) {
		fl_alert(_("%s %d Error #%d : %s."), __FILE__, __LINE__, err, sqlite3_errmsg(db));
	}
	sqlite3_finalize(pStmt);
}

void Position::load(void)
{
	sqlite3 *db;
	int err;
	sqlite3_stmt *pStmt = NULL;
	char *zSql = NULL;
	const char *zTail = NULL;

	db = ((Document*)parent()->parent())->db;

	zSql = "SELECT  "
		"line, "
		"type, "
		"pos, "
		"art, "
		"desc, "
		"tva, "
		"quantity, "
		"unit, "
		"price, "
		"pc, "
		"total"
		" FROM position;";
	err = sqlite3_prepare(db, zSql, -1, &pStmt, &zTail); 
	if (err == SQLITE_OK) {
		//sqlite3_bind_text(pStmt, 1, buffer, -1, SQLITE_STATIC);
		do {
			err = sqlite3_step(pStmt);
			if (err == SQLITE_ROW) {
				const char *ptr;
				int i;
				int lines = 0;
				for (i = 0; i <= NB_COL; i++) { 
					ptr = (const char *)sqlite3_column_text(pStmt, i);
					if (!ptr) ptr = "";
					switch (i) {
					case 0:
						lines = atol(ptr);
						while (lines >= rows()) {
							add_rows();
						}
						break;
					case 1:
						values[lines][TYPE_COL][0] = ptr[0];
						break;
					case 2:
						values[lines][POS_COL] = strdup(ptr);
						break;
					case 3:
						values[lines][ART_COL] = strdup(ptr);
						break;
					case 4:
						values[lines][DESC_COL] = strdup(ptr);
						break;
					case 5:
						values[lines][TVA_COL] = strdup(ptr);
						break;
					case 6:
						values[lines][QUANTITY_COL] = strdup(ptr);
						break;
					case 7:
						values[lines][UNIT_COL] = strdup(ptr);
						break;
					case 8:
						values[lines][PRICE_COL] = strdup(ptr);
						break;
					case 9:
						values[lines][PC_COL] = strdup(ptr);
						break;
					case 10:
						values[lines][TOTAL_COL] = strdup(ptr);
						break;
					}
				}
			}
		} while (err == SQLITE_ROW); 
	}

	if (err != SQLITE_DONE && err != SQLITE_OK) {
		fl_alert(_("%s %d Error #%d : %s."), __FILE__, __LINE__, err, sqlite3_errmsg(db));
	}
	sqlite3_finalize(pStmt);

}

PositionInput::PositionInput(int X, int Y, int W, int H) :
	Fl_Multiline_Input(X, Y, W, H)
{
	type(FL_MULTILINE_INPUT_WRAP);
}

int PositionInput::handle(int e)
{
	if (e == FL_KEYBOARD && Fl::e_length == 1 &&
		Fl::e_text[0] == '\r' &&
		!(Fl::event_state() & FL_ALT)) 
	{
		do_callback();
		return 1;
	}
	return Fl_Multiline_Input::handle(e);
}

void Position::print_new_page(PrintContext *ctx)
{
	ctx->tk->pop_clip();
	ctx->end_page();
	ctx->page_num++;
	ctx->y = 0;
	ctx->x = 0;
	ctx->start_page();
	ctx->tk->clip(0, 0, ctx->user_width, ctx->user_height);
}

void Position::print_head(PrintContext *ctx)
{
	int i;
	int x, y;
	fl_color(FL_BLACK);
	fl_font(hfont, hsize);
	
	if (ctx->y + 3 * fl_height() > ctx->user_height) {
		print_new_page(ctx);
	}

	x = ctx->x = 0;
	y = ctx->y = ctx->y + fl_height() - fl_descent();;
	for (i = 0; i < NB_COL; i++) {
		if (colw[i] > 0) {
			if (colt[i]) {
				int w, l;
				l = strlen(colt[i]);
				w = 0;
				switch (i) {
				case TVA_COL:
				case UNIT_COL:
				case PRICE_COL:
				case PC_COL:
				case TOTAL_COL:
					w = colw[i] - (int)fl_width(colt[i], l);
				}
				ctx->tk->draw(colt[i], l, x + w, y);
			}
			x += colw[i] + 3;
		}
	}
	ctx->y += fl_descent();
	ctx->tk->rectf(0, ctx->y, ctx->user_width, hsize / 10.0);
	ctx->y += hsize / 10 + 3;
}

void Position::print_section(PrintContext *ctx, int line, int *end)
{
	int x, y, i;

	fl_font(sfont, ssize);	

	if (ctx->y + fl_height() > ctx->user_height) {
		insert_line_before(line, END_PAGE_CL);
		line++;
		*end++;
		print_new_page(ctx);
		print_head(ctx);
	}

	fl_color(FL_BLACK);
	fl_font(sfont, ssize);	

	x = ctx->x = 0;
	y = ctx->y = ctx->y + fl_height() - fl_descent();;
	for (i = 0; i < NB_COL; i++) {
		if (colw[i] > 0) {
			const char *str = values[line][i];
			if (i == DESC_COL) str = values[line][ART_COL];
			if (str) {
				int w, l;
				l = strlen(str);
				w = 0;
				switch (i) {
				case TVA_COL:
				case QUANTITY_COL:
				case PRICE_COL:
				case PC_COL:
				case TOTAL_COL:
					w = colw[i] - (int)fl_width(str, l);
				}
				if (i == POS_COL) {
					ctx->tk->draw(str, l, x + w, y);
				} else if (i == DESC_COL) {
					int h;
					h = ctx->tk->draw_multi(str, 
						x, y, colw[ART_COL] + 
							colw[TVA_COL] +
							colw[QUANTITY_COL] + 
							colw[UNIT_COL] + 
							colw[PRICE_COL] + 
							colw[PC_COL], 0);
					h -= y;
					if (h > fl_height()) {
						y += h - fl_height();
						ctx->y = y;
					}	
				}
			}
			x += colw[i] + 3;
		}
	}
	ctx->y += fl_descent();
}

void Position::print_sub_total(PrintContext *ctx, int line, int *end)
{
	int x, y, i;

	fl_font(tfont, tsize);	

	if (ctx->y + fl_height() + 6> ctx->user_height) {
		insert_line_before(line, 'd');
		line++;
		*end++;
		print_new_page(ctx);
		print_head(ctx);
	}
	fl_color(FL_BLACK);
	fl_font(tfont, tsize);

	x = ctx->x = 0;
	y = ctx->y = ctx->y + fl_height() - fl_descent() + 6;
	for (i = 0; i < NB_COL; i++) {
		if (colw[i] > 0) {
			const char *str = values[line][i];
			if (str) {
				int w, l;
				l = strlen(str);
				w = 0;
				switch (i) {
				case TVA_COL:
				case QUANTITY_COL:
				case UNIT_COL:
				case PC_COL:
				case TOTAL_COL:
					w = colw[i] - (int)fl_width(str, l);
				}
				if (i == POS_COL) {
					ctx->tk->draw(str, l, x + w, y);
				} else if (i == TOTAL_COL) {
					ctx->tk->draw(str, l, x + w, y);
					ctx->tk->rectf(x, 
						ctx->y - fl_height() +
							fl_descent() - 2, 
						w, tsize / 10.0);
				} else if (i == DESC_COL) {
					int h;
					h = ctx->tk->draw_multi(str, 
						x, y, colw[DESC_COL], 0);
					h -= y;
					if (h > fl_height()) {
						y += h - fl_height();
						ctx->y = y;
					}	
				}
			}
			x += colw[i] + 3;
		}
	}
	ctx->y += fl_descent();
	
}

void Position::print_line(PrintContext *ctx, int line, int *end)
{
	int i;
	int x, y;

	fl_font(nfont, nsize);	

	if (ctx->y + fl_height() > ctx->user_height) {
		insert_line_before(line, 'd');
		line++;
		*end++;
		print_new_page(ctx);
		print_head(ctx);
	}

	fl_color(FL_BLACK);
	fl_font(nfont, nsize);	

	x = ctx->x = 0;
	y = ctx->y = ctx->y + fl_height() - fl_descent();;
	for (i = 0; i < NB_COL; i++) {
		if (colw[i] > 0) {
			const char *str = values[line][i];
			if (str) {
				int w, l;
				l = strlen(str);
				w = 0;
				switch (i) {
				case TVA_COL:
				case QUANTITY_COL:
				case PRICE_COL:
				case PC_COL:
				case TOTAL_COL:
					w = colw[i] - (int)fl_width(str, l);
				}
				if (i != DESC_COL) {
					ctx->tk->draw(str, l, x + w, y);
				} else {
					int h;
					h = ctx->tk->draw_multi(str, 
						x, y, colw[DESC_COL], 0);
					h -= y;
					if (h > fl_height()) {
						y += h - fl_height();
						ctx->y = y;
					}	
				}
			}
			x += colw[i] + 3;
		}
	}
	ctx->y += fl_descent();
}

void Position::print(PrintContext *ctx)
{
	PageSetup *stp = ((Document*)parent()->parent())->pagesetup;
	int n, i;

	if (stp->changed_ != last_changed_pagesetup) {
		int i, w;
		last_changed_pagesetup = stp->changed_;
		w = 0;
		for (i = 0; i < NB_COL; i++) {
			char buf[128];
			snprintf(buf, 128, "invoice_col%d_w", i);
			colw[i] = stp->get_int(buf);
			colw[i] = colw[i] * 720 / 254;
			w += colw[i];
			if (colw[i]) w += 3;
			snprintf(buf, 128, "invoice_col%d_t", i);
			colt[i] = stp->get_string(buf);
		}
		sub_tot = stp->get_string("invoice_sub_tot");
		colw[2] = ctx->user_width - w;
		stp->get_font("invoice_col_head_font", &hfont, &hsize, &hunder);
		stp->get_font("invoice_normal_font", &nfont, &nsize, &nunder);
		stp->get_font("invoice_subtot_font", &tfont, &tsize, &tunder);
		stp->get_font("invoice_sect_font", &sfont, &ssize, &sunder);
	}

	n = rows() - 1;
	while (n >= 0 && values[n][TYPE_COL][0] == 'a' &&
		(!values[n][DESC_COL] || !values[n][DESC_COL][0]))
	{
		n--;
	}
	if ( n < 0) return;

	print_head(ctx);
	
	for (i = 0; i <= n; i++) {
		switch (values[i][TYPE_COL][0]) { 
		case NORMAL_CL:
			print_line(ctx, i, &n);
			break;
		case BEG_SEC_CL:
			print_section(ctx, i, &n);
			break;
		case BREAK_CL:
			print_new_page(ctx);
			if (i < n) print_head(ctx);
			break;
		case END_SEC_CL:
			print_sub_total(ctx, i, &n);
		}		
	}
}

//
// End of "$Id: $".
//

