//
// "$Id: $"
//
// Customer. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//

#ifndef POSITION_H
#define POSITION_H

#include <FL/Fl_Table.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Menu_Button.H>

#include "Printing.h"

class PositionInput : public Fl_Multiline_Input {
public:
	PositionInput(int X, int Y, int W, int H);
	int handle(int e);
};

class Position : public Fl_Table {
public:
	int s_left, s_top, s_right, s_bottom;
	int row_edit, col_edit;
	char ***values;
	char *col_names[10];
	PositionInput *input;
	Fl_Menu_Button *menu;
	int changed_;
	char draw_id;

	const char *sub_tot;
	int colw[10];
	char *colt[10];
	Fl_Font hfont;
	Fl_Font nfont;
	Fl_Font tfont;
	Fl_Font sfont;
	int hsize;
	int nsize;
	int tsize;
	int ssize;
	int hunder;
	int nunder;
	int tunder;
	int sunder;
	int last_changed_pagesetup;

	Position(int X, int Y, int W, int H);
	~Position();

	static void event_callback1(Fl_Widget*, void*);
	static void input_callback1(Fl_Widget*, void*);
	static void menu_callback1(Fl_Widget*, void*);
	void input_callback(void);
	void menu_callback(void);
	void event_callback(void);
	void draw_cell(TableContext context, int=0, 
		int=0, int=0, int=0, int=0, int=0);

	void add_rows(void);
	void make_room(int start, int nb);
	void insert_line_before(int line, char type);

	void init(void);
	void load(void);
	void set(int line);
	void paste(void);
	void select(void);
	void cut(int paste = 0);

	void begin_transaction(void);
	void end_transaction(void);

	void recompute(void);

	void print(PrintContext *ctx);
	void print_head(PrintContext *ctx);
	void print_new_page(PrintContext *ctx);
	void print_line(PrintContext *ctx, int line, int *end);
	void print_section(PrintContext *ctx, int line, int *end);
	void print_sub_total(PrintContext *ctx, int line, int *end);
};


#endif // POSITION_H

//
// End of "$Id: $".
//

