//
// "$Id: $"
//
// Pref dialog. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "PrefDialog.h"
#include "TheApp.h"
#include <FL/Fl.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Choice.H>

PrefDialog::PrefDialog(int X, int Y, int W, int H) : Fl_Group(X, Y, W, H)
{
	res = new Fl_Button(X + W - 2, Y + H - 2, 2, 2);
	res->hide();

	constructor(X, Y, W, H);
}

PrefDialog::PrefDialog(int W, int H) : Fl_Group(0, 0, W, H)
{
	int Y, X;
	X = 5;
	Y = 5;
	Fl_Group::end();

	wi = new Fl_Window(W, H);
	
	wi->add(this);
	position(0,0);
	wi->resizable(this);
	wi->size_range(200, 200);
	
	begin();
	label(_("Pref"));
	save = new Fl_Button(X, Y, 80, 25, _("Sauver"));
	
	abort = new Fl_Button(X + 90, Y, 80, 25, _("Annuler"));
	res = new Fl_Button(X + W - 2, Y + H - 2, 2, 2);
	res->hide();
	Y += 35;



	constructor(X - 5, Y - 5, W, H - 35);
}

void PrefDialog::constructor(int X, int Y, int W, int H)
{
	int OX, OY;
	OX = X;
	OY = Y;
	X += 5;
	Y += 5;

	align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	labelfont(FL_HELVETICA_BOLD);

	scroll = new Fl_Scroll(OX, Y, W, H - 5);
	res1 = new Fl_Button(OX, Y, 20, 20);
	res1->hide();
	scroll->end();
	scroll->box(FL_DOWN_BOX);

	ypos = 0;
	resizable(res);

	items = NULL;
	nb_items = 0;
	alloc_items = 0;

	cw1 = cw2 = 120; 
	end();
}



PrefDialog::~PrefDialog(void)
{
	while (nb_items > 0) {
		nb_items--;
		free(items[nb_items]);
	}
	free(items);
}

PrefDialogItem *PrefDialog::grow()
{
	if (nb_items >= alloc_items) {
		alloc_items += 8;
		items = (PrefDialogItem**) realloc(items, 
			sizeof(PrefDialogItem*) * alloc_items);	
	}
	items[nb_items] = (PrefDialogItem*) malloc(sizeof(PrefDialogItem)); 
	return items[nb_items++];
}
	
void PrefDialog::add_text(const char *id, const char *name, char **txt)
{
	PrefDialogItem *d;

	d = grow();
	
	scroll->begin();
	d->w = new Fl_Input(cw1, scroll->y() + ypos, cw2, 22, name);
	if (txt) ((Fl_Input*)d->w)->value(*txt); 
	scroll->end();

	ypos += 25;
	d->sql_id = id;
	d->data = NULL;
	d->name = name;
	d->type = TEXT_;
	d->value = txt;

}

void PrefDialog::add_path(const char *id, const char *name, char **pth)
{
}

void PrefDialog::add_int(const char *id, const char *name, int *i)
{
}

void PrefDialog::add_int64(const char *id, const char *name, i64 *i)
{
}

void PrefDialog::add_double(const char *id, const char *name, double *d)
{
}

void PrefDialog::add_choice(const char *id, const char *name, int *ret, Fl_Menu_Item *menu)
{
}

void PrefDialog::add_check(const char *id, const char *name, int *ret)
{
}

int PrefDialog::run(void)
{

	scroll->position(-3, -3);
	wi->label(label());
	wi->show();
	scroll->take_focus();
	while (1) {
		Fl_Widget *o = Fl::readqueue();
		if (!o) Fl::wait();
		else if (o == abort || o == wi) { 
			wi->hide(); 
		
			return 0;
		} else if (o == save) { 
			wi->hide();
			sync(); 
			return 1;
		}
	}
	return 0;		
}

void PrefDialog::sync()
{
	int i;
	for (i = 0; i < nb_items; i++) {
		switch (items[i]->type) {
		case TEXT_:
			*((char**)(items[i]->value)) = strdup(
				((Fl_Input*)items[i]->w)->value());
			break;
		case PATH_:
			break;
		case INT_:
			*((int*)(items[i]->value)) = atol(
				((Fl_Int_Input*)items[i]->w)->value());
			break;
		case DOUBLE_:
		case CHOICE_:
		case INT64_:		
		default:
			break;
		}
	}
}


void PrefDialog::ended()
{
	end();
	//init_sizes();
	scroll->position(-3, -3);

}


//
// End of "$Id: $".
//

