//
// "$Id: $"
//
// Application main object. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//

#ifndef PREFDIALOG_H
#define PREFDIALOG_H

#include <FL/Fl_Window.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Scroll.H>
#include "global.h"

enum PrefDialogItemType {
	TEXT_,
	PATH_,
	INT_,
	INT64_,
	DOUBLE_,
	CHOICE_,
	CHECK_,
};
	
struct PrefDialogItem {
	int type;
	const char *sql_id;
	const char *name;
	void *value;
	void *data;
	Fl_Widget *w;
};

class PrefDialog : public Fl_Group {
public:
	Fl_Window *wi;
	Fl_Button *abort;
	Fl_Button *save;
	Fl_Scroll *scroll;
	Fl_Button *res;
	Fl_Button *res1;

	PrefDialogItem **items;
	int nb_items;
	int alloc_items;

	int ypos;
	int cw1;
	int cw2;
	
	PrefDialog(int X, int Y, int W, int H);
	PrefDialog(int W, int H);
	~PrefDialog(void);
	
	void constructor(int X, int Y, int W, int H);

	PrefDialogItem *grow(void);

	void add_text(const char *id, const char *name, char **txt);
	void add_path(const char *id, const char *name, char **pth);
	void add_int(const char *id, const char *name, int *i);
	void add_int64(const char *id, const char *name, i64 *i);
	void add_double(const char *id, const char *name, double *d);
	void add_choice(const char *id, const char *name, int *ret, Fl_Menu_Item *menu);
	void add_check(const char *id, const char *name, int *ret);
	void ended(void);
	void col_width(int c1, int c2) {cw1 = c1; cw2 = c2;}

	int run(void);
	void sync(void);	
};


#endif // PREFDIALOG_H

//
// End of "$Id: $".
//

