//
// "$Id: $"
//
// Print kit. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "Printing.H"
#include <FL/fl_draw.H>


void PrintKit::clip(int x, int y, int w, int h){
	if (ctx->page_num < ctx->from_page || 
		ctx->page_num > ctx->to_page)
	{
		return;
	} 
	convert(x,y,w,h);
	switch (ctx->draw) {
	case 1:
		ogc = fl_gc;
		fl_gc = ctx->gc;
		fl_clip(x,y,w,h);
		fl_gc = ogc;
		break;
	case 2:
		fl_clip(x,y,w,h);
	}			
}

void PrintKit::pop_clip()
{
	if (ctx->page_num < ctx->from_page || 
		ctx->page_num > ctx->to_page)
	{
		return;
	} 
	switch (ctx->draw) {
	case 1:
		ogc = fl_gc;
		fl_gc = ctx->gc;
		fl_pop_clip();
		fl_gc = ogc;
		break;
	case 2:
		fl_pop_clip();
	}	
}

void PrintKit::rectf(double x, double y, double w, double h)
{
	if (ctx->page_num < ctx->from_page || 
		ctx->page_num > ctx->to_page)
	{
		return;
	} 
	convert(x,y,w,h);
	switch (ctx->draw) {
	case 1:
		ogc = fl_gc;
		fl_gc = ctx->gc;
		fl_color(fl_color());
		fl_rectf((int)x,(int)y,(int)w,(int)h);
		fl_gc = ogc;
		break;
	case 2:
		fl_rectf((int)x,(int)y,(int)w,(int)h);
	}			
}

void PrintKit::draw(const char* s, int n, int x, int y)
{
	int a;
	if (ctx->page_num < ctx->from_page || 
		ctx->page_num > ctx->to_page)
	{
		return;
	} 
	convert(x,y);
	switch (ctx->draw) {
	case 1:
		ogc = fl_gc;
		fl_gc = ctx->gc;
		a = fl_size();
		fl_color(fl_color());
		fl_font(fl_font(), (int)(a * ctx->scale));	
		fl_draw(s,n,x,y);
		fl_font(fl_font(), a);
		fl_gc = ogc;
		break;
	case 2:
		a = fl_size();
		fl_font(fl_font(), (int)(a * ctx->scale));	
		fl_draw(s,n,x,y);
		fl_font(fl_font(), a);
	}	
}

void PrintKit::rtl_draw(const char* s, int n, int x, int y)
{
	int a;
	if (ctx->page_num < ctx->from_page || 
		ctx->page_num > ctx->to_page)
	{
		return;
	} 
	convert(x,y);
	switch (ctx->draw) {
	case 1:
		ogc = fl_gc;
		fl_gc = ctx->gc;
		a = fl_size();
		fl_color(fl_color());
		fl_font(fl_font(), (int)(a * ctx->scale));	
		fl_rtl_draw(s,n,x,y);
		fl_font(fl_font(), a);
		fl_gc = ogc;
		break;
	case 2:
		a = fl_size();
		fl_font(fl_font(), (int)(a * ctx->scale));	
		fl_rtl_draw(s,n,x,y);
		fl_font(fl_font(), a);
	}	
}

void PrintKit::draw_image(int x,int y,int W,int H, int bw, int bh, int d, int ld, const uchar*buf)
{
	uchar *new_array, *new_ptr;
	const uchar *old_ptr;
	int c,sy,dx, dy, xerr, yerr, xmod, ymod, xstep, ystep;	

	if (ctx->page_num < ctx->from_page || 
		ctx->page_num > ctx->to_page)
	{
		return;
	} 

	if (ctx->draw < 1 || !buf) return;

	convert(x,y,W,H);

	if (bw == W && bh == H) {
		switch (ctx->draw) {
		case 1:
			ogc = fl_gc;
			fl_gc = ctx->gc;
			fl_draw_image(buf,x,y,W,H,d, ld);
			fl_gc = ogc;
			break;
		case 2:
			fl_draw_image(buf,x,y,W,H,d, ld);
		}				
		return;
	}

	if (W <= 0 || H <= 0) return;

	xmod   = bw % W;
	xstep  = (bw / W) * d;
	ymod   = bh % H;
	ystep  = bh / H;

	new_array = new uchar [W * H * d];
 
	// Scale the image 
	for (dy = H, sy = 0, yerr = H, new_ptr = new_array; dy > 0; dy --) {
		for (dx = W, xerr = W, old_ptr = buf + sy * (bw * d + ld);
			dx > 0; dx --) 
		{
			for (c = 0; c < d; c ++) *new_ptr++ = old_ptr[c];

			old_ptr += xstep;
			xerr    -= xmod;

			if (xerr <= 0) {
				xerr    += W;
				old_ptr += d;
			}
		}

		sy   += ystep;
		yerr -= ymod;
		if (yerr <= 0) {
			yerr += H;
			sy ++;
		}
	}
	switch (ctx->draw) {
	case 1:
		ogc = fl_gc;
		fl_gc = ctx->gc;
		fl_draw_image(new_array,x,y,W,H,d, ld);
		fl_gc = ogc;
		break;
	case 2:
		fl_draw_image(new_array,x,y,W,H,d, ld);
	}		
	

	delete[] new_array;
}

int PrintKit::draw_multi(const char* s, int x, int y, int w, int under)
{
	char buf[2048];
	const char *next = s;
	int len = 0;
	double width = 0;

	do {
		if (y + fl_height() > ctx->user_height) {
			pop_clip();
			ctx->end_page();
			ctx->page_num++;
			ctx->start_page();
			clip(0, 0, ctx->user_width, ctx->user_height);
		}
		next = fl_expand_text(next, buf, 2048, w, len, width, 1, 0); 
		if (len > 0) {
			draw(buf, len, x, y);
			if (under == 1) {
				ctx->tk->rectf(x, y + 1, width, fl_size() / 10.0);				
			}
			y += fl_height();
		}
	} while (len > 0);

	return y;
}

//
// End of "$Id: $".
//

