//
// "$Id: $"
//
// Printing dialog. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//



#include "global.h"
#include "Printing.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/fl_draw.H>

static PrintAbort *pabort = NULL;

PrintAbort::PrintAbort() : Fl_Window (100, 80, _("Impression en cours"))
{
	b = new Fl_Button(20, 20, 80, 40, _("Annuler"));
	b->callback(abort_cb, this);	
	set_modal();
}

void PrintAbort::abort_cb(Fl_Widget *e, void *d) 
{
	((PrintAbort*)d)->hide();	
}

int PrintAbort::no()
{
	return shown();
}

void PrintAbort::run()
{
	show();
	Fl::check();
	Fl::flush();
}

void PrintAbort::stop()
{
	hide();
}

BOOL CALLBACK AbortProc(HDC dc, int err) 
{
	if (SP_OUTOFDISK == err || err == 0) return TRUE;
	pabort->stop();
	return FALSE;
}

PrintContext::PrintContext(void)
{
	x = y = 0;
	dx = dy = 0;

	landscape = 0;

	abort = NULL;
	data = NULL;
	from_page = 1;
	to_page = 1;
	nb_copies = 1;
	nb_page = 1;

	doc_info.cbSize = sizeof(DOCINFO);
	doc_info.lpszDocName = "OksiD";
	doc_info.lpszOutput = NULL;
	doc_info.fwType = 0;

	ZeroMemory(&pd, sizeof(pd));

	page_width = 595;
	page_height = 842;
	margin_top = 1 * 7200 / 254;
	margin_left = (2 * 7200 + 3600) / 254;
	margin_right = 2 * 7200 / 254;
	margin_bottom = 2 * 7200 / 254;

	scale = 1.0;
	printer_scale = 1.0;

	gc = 0;
	tk = new PrintKit(this);
}

void PrintContext::user_space()
{
	if (landscape) {
		user_width = page_height - margin_top - margin_bottom;
		user_height = page_width - margin_left - margin_right;
		user_left = margin_top;
		user_top = margin_left;
	} else {
		user_height = page_height - margin_top - margin_bottom;
		user_width = page_width - margin_left - margin_right;
		user_top = margin_top;
		user_left = margin_left;
	}
}

void PrintContext::get_gc(int dont_ask)
{
	pd.lStructSize = sizeof(pd);
	pd.hwndOwner   = 0;	   
	pd.hDevNames   = NULL;     
	pd.Flags       = PD_PAGENUMS | PD_RETURNDC | PD_NOSELECTION;
	if (dont_ask) {
		pd.Flags = PD_RETURNDEFAULT;
		pd.hDevMode = NULL;
	} 
	pd.nCopies     = 1;
	if (dont_ask) {
		pd.nFromPage   = from_page; 
		pd.nToPage     = to_page; 
	} else {
		pd.nFromPage   = 1; 
		pd.nToPage     = nb_page;
	}
	pd.nMinPage    = 1; 
	pd.nMaxPage    = nb_page; 

	if (pd.hDC) DeleteDC(pd.hDC);
	pd.hDC = NULL;

	if (PrintDlg(&pd)==TRUE) {
		double s;
		DEVMODE *dm = (DEVMODE *)GlobalLock(pd.hDevMode);
		if (dm) {

			if (dm->dmFields & DM_PAPERSIZE) {
				switch (dm->dmPaperSize) {
				case DMPAPER_A5:
					page_width = 148 * 720 / 254;
					page_height = 210 * 720 / 254;
					break;				
				case DMPAPER_A3:
					page_width = 297 * 720 / 254;
					page_height = 420 * 720 / 254;
					break;
				case DMPAPER_A4SMALL:
				case DMPAPER_A4:
					page_width = 210 * 720 / 254;
					page_height = 297 * 720 / 254;
					break;
				case DMPAPER_LETTER:
				case DMPAPER_LETTERSMALL:
					page_width = 8 * 72 + 72 / 2;
					page_height = 11 * 72;
					break;
				case DMPAPER_LEGAL:
					page_width = 8 * 72 + 72 / 2;
					page_height = 14 * 72;
					break;
				}
			} else if (dm->dmFields & DM_PAPERLENGTH) {
				page_width = dm->dmPaperWidth * 72 / 254;
				page_height = dm->dmPaperLength  * 72 / 254;
			}  
			if (dm->dmFields & DM_ORIENTATION) {
				landscape = (dm->dmOrientation == DMORIENT_LANDSCAPE);
			}
			GlobalUnlock(dm);
		}
		printer_scale = GetDeviceCaps(pd.hDC, PHYSICALWIDTH) / (double)page_width;
		s = GetDeviceCaps(pd.hDC, PHYSICALHEIGHT) / (double)page_height;
		if (s < printer_scale) printer_scale = s;
		user_space();	
	}
	gc = pd.hDC;

	if (!dont_ask) {
		from_page = pd.nFromPage; 
		to_page = pd.nToPage; 
	}
}

void PrintContext::start_doc(void)
{
	if (!gc) get_gc(1);
	SetAbortProc(gc, AbortProc);
	StartDoc(gc, &doc_info);
	page_num = 0;
}

void PrintContext::end_doc(void)
{
	EndDoc(gc);
}

void PrintContext::start_page(void)
{
	if (page_num < from_page || 
		page_num > to_page)
	{
		return;
	}  
	if (draw == 1) {
		StartPage(gc);
		SetTextAlign(gc, TA_BASELINE|TA_LEFT);
    		SetBkMode(gc, TRANSPARENT);
	}
	doc->foot_head(this);
}

void PrintContext::end_page(void)
{
	if (draw != 1) return; 
	if (page_num < from_page || 
		page_num > to_page)
	{
		return;
	}  
	EndPage(gc);
}


Printing::Printing() 
{
	ctx = new PrintContext();
	if (!pabort) pabort = new PrintAbort();
	ctx->abort = pabort;
	dlg =  new PrintDialog(620, 400);
}

void Printing::print(PrintDocument *doc)
{
	if (ctx->from_page < 1) return;
	measure(doc);
	dlg->print_widget->ctx = ctx;
	dlg->print();
}

void Printing::preview(PrintDocument *doc)
{
	measure(doc);
	dlg->run(ctx);
	doc->margins(ctx->page_width, ctx->page_height, ctx->margin_left, 
		ctx->margin_top, ctx->margin_right, ctx->margin_bottom,
		ctx->landscape);
}

void Printing::measure(PrintDocument *doc)
{
	if (!ctx->gc) ctx->get_gc(1);
	ctx->doc = doc;
	ctx->draw = 0;
	ctx->scale = 1.0;
	ctx->nb_page = 0;
	ctx->user_space();
	doc->print(ctx);
}

PrintWidget::PrintWidget(int X, int Y, int W, int H)
: Fl_Widget(X, Y, W, H)
{

}

void PrintWidget::resize(PrintContext *c)
{
	ctx = c;
	if (ctx->landscape) {
		Fl_Widget::resize(x(), y(), 
			(int)(ctx->page_height * ctx->scale) + 7, 
			(int)(ctx->page_width * ctx->scale) + 7);
	} else {
		Fl_Widget::resize(x(), y(), 
			(int)(ctx->page_width * ctx->scale) + 7, 
			(int)(ctx->page_height * ctx->scale) + 7);
	}
}

void PrintWidget::draw()
{
	ctx->dx = x() + 2;
	ctx->dy = y() + 2;
	ctx->mx = x() + w() - 7;
	ctx->my = y() + h() - 7;

	fl_color(FL_WHITE);
	fl_rectf(x(), y(), w(), h());
	fl_color(FL_BLACK);
	fl_rectf(x(), y(), w() - 5, 2);
	fl_rectf(x(), y() + h() - 5, w(), 5);
	fl_rectf(x(), y(), 2, h() - 5);
	fl_rectf(x() + w() - 5, y(), 5, h());
	fl_color(color());
	fl_rectf(x(), y() + h() - 3, 3, 3);
	fl_rectf(x() + w() - 3, y(), 3, 3);

	fl_clip(ctx->dx, ctx->dy, ctx->mx - ctx->dx, ctx->my - ctx->dy); 
	fl_color(FL_BLACK);
	ctx->draw = 2;
	ctx->dx = x() + 2;
	ctx->dy = y() + 2;
	ctx->doc->print(ctx);
	fl_pop_clip();
}

enum {
	Z_25 = 1,
	Z_50,
	Z_100,
	Z_200,
	B_PRINT,
	B_SETTINGS,
	B_MARGIN,
	B_ABORT,
};

void PrintDialog::event_cb(Fl_Widget *w, void *d)
{
	PrintDialog *p = (PrintDialog*)w->parent();

	if ((int)d < 0) {
		p->print_widget->ctx->from_draw = p->print_widget->ctx->to_draw = -(int)d;
		p->redraw();
		return;
	}
	switch((int) d) {
	case B_PRINT:
		p->hide();
		p->print();
		break;
	case B_SETTINGS:
		p->print_widget->ctx->get_gc();
		p->print_widget->resize(p->print_widget->ctx);
		p->scroll->position(0,0);
		p->redraw();
		break;
	case B_MARGIN:
		break;
	case B_ABORT:
		p->hide();
		break;
	case Z_25:
		p->print_widget->ctx->scale = 0.25;
		p->print_widget->resize(p->print_widget->ctx);
		p->scroll->position(0,0);
		p->redraw();
		break;
	case Z_50:
		p->print_widget->ctx->scale = 0.50;
		p->print_widget->resize(p->print_widget->ctx);
		p->scroll->position(0,0);
		p->redraw();
		break;
	case Z_100:
		p->print_widget->ctx->scale = 1.0;
		p->print_widget->resize(p->print_widget->ctx);
		p->scroll->position(0,0);
		p->redraw();
		break;
	case Z_200:
		p->print_widget->ctx->scale = 2.0;
		p->print_widget->resize(p->print_widget->ctx);
		p->scroll->position(0,0);
		p->redraw();
		break;


	}
}

PrintDialog::PrintDialog(int W, int H) : Fl_Window(W, H)
{
	int X = 2;
	int Y = 2;

	label(_("Impression"));
	b_print = new Fl_Button(X, Y, 90, 22, _("Imprimer"));
	b_print->callback(event_cb, (void*)B_PRINT);
	X += 92;
	b_settings = new Fl_Button(X, Y, 90, 22, _("Configuration"));
	b_settings->callback(event_cb, (void*)B_SETTINGS);
	X += 92;
	b_margin = new Fl_Button(X, Y, 80, 22, _("Marges"));
	b_margin->callback(event_cb, (void*)B_MARGIN);
	X += 92;
	c_page = new Fl_Choice(X + 50, Y, 60, 22, _("Page"));
	X += 112;
	c_zoom = new Fl_Choice(X + 50, Y, 60, 22, _("Zoom"));
	X += 112;
	b_abort = new Fl_Button(X, Y, 80, 22, _("Fermer"));
	b_abort->callback(event_cb, (void*)B_ABORT);
	X += 92;

	m_page = new Fl_Menu_Item[4];
	m_page->text = NULL;
	m_page->add(_("1"), 0, event_cb, (void*)-1, 0);
	c_page->menu(m_page);

	m_zoom = new Fl_Menu_Item[4];
	m_zoom->text = NULL;
	m_zoom->add(_("25%"), 0, event_cb, (void*)Z_25, 0);
	m_zoom->add(_("50%"), 0, event_cb, (void*)Z_50, 0);
	m_zoom->add(_("100%"), 0, event_cb, (void*)Z_100, 0);
	m_zoom->add(_("200%"), 0, event_cb, (void*)Z_200, 0);
	c_zoom->menu(m_zoom);
	c_zoom->value(2);

	scroll = new Fl_Scroll(2, 27, W - 2, H - 27);
	print_widget = new PrintWidget(2, 27, W, H);
	scroll->end();
	scroll->box(FL_FLAT_BOX);
	scroll->resizable(NULL);
	b_res = new Fl_Button(W - 2, H - 2, 2, 2);
	b_res->hide();
	resizable(b_res);
	init_sizes();
	size_range(200, 200);
}

void PrintDialog::run(PrintContext *ctx)
{
	int i;
	char buf[256];
	print_widget->resize(ctx);
	delete[] m_page;

	if (ctx->nb_page < 1) ctx->nb_page = 1;
	m_page = new Fl_Menu_Item[ctx->nb_page + 2];
	m_page->text = NULL;
		
	for (i=0; i < ctx->nb_page; i++) {
		snprintf(buf, 255, "%d", i + 1);
		m_page->add(buf, 0, event_cb, (void*)-(i + 1), 0);
	}
	c_page->menu(m_page);
	c_page->value(0);

	show();
}

void PrintDialog::print()
{
	HDC ogc;
	PrintContext *ctx;
	ctx = print_widget->ctx;
	ctx->from_draw = ctx->from_page;
	ctx->to_draw = ctx->to_page;
	ctx->draw = 1;
	ctx->dx = 0;
	ctx->dy = 0;
	pabort->run();

	ctx->start_doc();

	ogc = fl_gc;
	fl_gc = ctx->gc;
	fl_push_no_clip();
	fl_gc = ogc;

	ctx->scale = ctx->printer_scale;	

	ctx->doc->print(ctx);
	ctx->end_doc();
	pabort->stop();

	ogc = fl_gc;
	fl_gc = ctx->gc;
	fl_pop_clip();
	fl_gc = ogc;

	ctx->scale = 1.0;
}

//
// End of "$Id: $".
//

