//
// "$Id: $"
//
// Customer. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//

#ifndef PRINTING_H
#define PRINTING_H

#include <FL/x.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Input.H>
#include <FL/fl_draw.H>

class PrintDocument;
class PrintKit;

class PrintAbort : public Fl_Window
{
public:
	Fl_Button *b;
	PrintAbort(void);
	static void abort_cb(Fl_Widget *e, void *d);
	int no(void);
	void run(void);
	void stop(void);
};




class PrintContext {
public:
	int x;
	int y;

	int dx;
	int dy;
	int mx;
	int my;

	int from_draw;
	int to_draw;

	HDC gc;
	PRINTDLG  pd;
	DOCINFO doc_info;
	PrintKit *tk;

	PrintDocument *doc;
	void *data;

	int page_num;

	int nb_page;
	int from_page;
	int to_page;
	int nb_copies;
	
	int page_height;
	int page_width;

	int user_top;
	int user_left;
	int user_height;
	int user_width;

	int foot_height;
	int head_height;
	int real_height;

	int margin_top;
	int margin_left;
	int margin_right;
	int margin_bottom;

	int landscape;

	double scale;
	double printer_scale;

	int draw;

	PrintAbort *abort;	

	PrintContext(void);
	virtual void start_doc();
	virtual void end_doc();
	virtual void start_page();
	virtual void end_page();
	virtual void get_gc(int dont_ask = 0);
	void user_space(void);
};

class PrintKit {
public:
	enum {
		DONT_DRAW = 0,
		GDI_DRAW,
		FLTK_DRAW,
		PDF_DRAW,
		PS_DRAW,
	};
	PrintContext *ctx;
	HDC ogc;
	inline void convert(double &x, double &y, double &w, double &h) {
		x = ((x + ctx->user_left)* ctx->scale) + ctx->dx + .5;
		y = ((y + ctx->user_top)* ctx->scale) + ctx->dy + .5;
		w = (w * ctx->scale) + .5;
		h = (h * ctx->scale) + .5;
	}
	inline void convert(int &x, int &y, int &w, int &h) {
		convert(x, y);
		w = (int)(w * ctx->scale);
		h = (int)(h * ctx->scale);
	}
	inline void convert(int &x, int &y) {
		x = (int)((x + ctx->user_left)* ctx->scale) + ctx->dx;
		y = (int)((y + ctx->user_top)* ctx->scale) + ctx->dy;
	}
	virtual void clip(int x, int y, int w, int h);
	virtual void pop_clip();
	virtual void rectf(double x, double y, double w, double h);
	virtual void draw(const char* s, int n, int x, int y);
	virtual void rtl_draw(const char* s, int n, int x, int y);
	virtual int draw_multi(const char* s, int x, int y, int w, int under);
	virtual void draw_image(int x,int y,int w,int h, int bw, int bh, int delta, int ldelta, const uchar*);
	PrintKit(PrintContext*c) {ctx = c;}
	virtual ~PrintKit(){};
};

class PrintDocument : public Fl_Group {
public:

	PrintDocument(int X, int Y, int W, int H) : Fl_Group(X,Y,W,H) {;}
	virtual void print(PrintContext *ctx) = 0;
	virtual void foot_head(PrintContext *ctx) = 0;
	virtual void margins(int w, int h, int left, int top, int right, int bottom, int land) = 0;
};

class PrintWidget : public Fl_Widget
{
public:
	PrintContext *ctx;

	PrintWidget(int X, int Y, int W, int H);	
	void draw(void);
	void resize(PrintContext *ctx);
};

class PrintMargin : public Fl_Window
{
public:
	PrintContext *ctx;

	Fl_Button *b_abort;
	Fl_Button *b_save;
	Fl_Input *i_top;
	Fl_Input *i_bottom;
	Fl_Input *i_left;
	Fl_Input *i_right;
	Fl_Choice *c_land;
	Fl_Menu_Item *m_land;

	PrintMargin(int W, int H);
	void run(PrintContext *ctx);
};

class PrintDialog : public Fl_Window
{
public:
	PrintMargin *margin_dlg;
	PrintWidget *print_widget;
	Fl_Scroll *scroll;
	Fl_Button *b_print;
	Fl_Button *b_abort;
	Fl_Button *b_margin;
	Fl_Button *b_settings;
	Fl_Button *b_res;
	Fl_Choice *c_page;
	Fl_Menu_Item *m_page;
	Fl_Choice *c_zoom;
	Fl_Menu_Item *m_zoom;

	PrintDialog(int W, int H);
	void run(PrintContext *ctx);
	static void event_cb(Fl_Widget *w, void *d);
	void print(void);
};

class Printing {
public:
	PrintContext *ctx;
	PrintDialog *dlg;

	Printing(void);
	void print(PrintDocument *doc);
	void preview(PrintDocument *doc);
	void measure(PrintDocument *doc);
};

#endif // PRINTING_H

//
// End of "$Id: $".
//

