//
// "$Id: $"
//
// Pref dialog. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "global.h"
#include "Settings.h"
#include <FL/Fl_File_Chooser.H>

static void win2unix(char *p)
{
#ifdef WIN32
	while (*p) {
		if (*p == '\\') *p = '/';
		p++;
	}
#endif
}

#ifdef WIN32
int fl_read_key(HKEY hkey, char *subkey, char *name, char *buffer, int len)
{

	LONG err;
	DWORD type;
	HKEY key;

	if (!subkey || !name || !buffer || len < 1) return ERROR_INVALID_DATA;

	buffer[0] = 0;

	if (fl_is_nt4()) {
		int l = strlen(subkey);
		xchar *sk = (xchar*) malloc((l + 1) * sizeof(xchar));
		sk[fl_utf2unicode((unsigned char*)subkey, l, sk)] = 0;
		err = RegOpenKeyW(hkey,	sk, &key);
		free(sk);
	} else {
		err = RegOpenKeyA(hkey,	fl_utf2mbcs(subkey), &key);	
	}
	if (err != ERROR_SUCCESS) return err;
 
	if (fl_is_nt4()) {
		unsigned long l = strlen(name);
		xchar *sk = (xchar*) malloc((l + 1) * sizeof(xchar));
		sk[fl_utf2unicode((unsigned char*)name, l, sk)] = 0;
		l = len / 5;
		xchar *buf = (xchar*) malloc(l * sizeof(xchar));
		buf[0] = 0;
		err = RegQueryValueExW( key, sk, 0L, &type, (BYTE*)buf, &l);
		if (type != REG_SZ) l = 0;
		buffer[fl_unicode2utf(buf, l, buffer)] = 0;
		free(sk);
		free(buf);
	} else {
		char *ptr;
		char *buf = (char*) malloc(len);
		unsigned long l = len;
		int i = 0;
		err = RegQueryValueExA(key, fl_utf2mbcs(name), 
			0L, &type, (BYTE*)buf, &l);
		if (type != REG_SZ) l = 0;
		buf[l] = 0;
		ptr = fl_mbcs2utf(buf);
		free(buf);
		while (ptr && *ptr && i < len - 1) { 
			buffer[i++] = *ptr;
			ptr++;
		}
		buffer[i] = 0;
	}
	RegCloseKey(key);
	return err;


}
#endif

#ifdef WIN32
int fl_write_key(HKEY hkey, char *subkey, char *name, char *buffer)
{

	LONG err;
	HKEY key;
	int len;

	if (!subkey || !name || !buffer) return ERROR_INVALID_DATA;

	len = strlen(buffer);

	if (fl_is_nt4()) {
		int l = strlen(subkey);
		xchar *sk = (xchar*) malloc((l + 1) * sizeof(xchar));
		sk[fl_utf2unicode((unsigned char*)subkey, l, sk)] = 0;
		err = RegCreateKeyW(hkey, sk, &key);
		free(sk);
	} else {
		err = RegCreateKeyA(hkey, fl_utf2mbcs(subkey), &key);	
	}
	if (err != ERROR_SUCCESS) return err;

	if (fl_is_nt4()) {
		unsigned long l = strlen(name);
		xchar *sk = (xchar*) malloc((l + 1) * sizeof(xchar));
		sk[fl_utf2unicode((unsigned char*)name, l, sk)] = 0;

		xchar *buf = (xchar*) malloc((len + 1)  * sizeof(xchar));
		l = fl_utf2unicode((unsigned char*)buffer, len, buf);
		buf[l] = 0;
		err = RegSetValueExW( key, sk, 0L, REG_SZ, (BYTE*)buf, l * 2);
		free(sk);
		free(buf);
	} else {
		char *buf;
		unsigned long l;
		buf = strdup(fl_utf2mbcs(buffer));
		l = strlen(buf);
		
		err = RegSetValueExA(key, fl_utf2mbcs(name), 
			0L, REG_SZ, (BYTE*)buf, l);
		
		free(buf);
	}
	RegCloseKey(key);
	return err;

}
#endif

Settings::Settings() : Fl_Preferences(USER, "O_ksi_D", "invoicing")
{
	app = new Fl_Preferences(this, 
#ifdef INVOICE
		"invoice");
#else
		"estimate");
#endif	

	data_path = NULL;
	tva_path = NULL;
	article_path = NULL;
	customer_path = NULL;
}

void Settings::init()
{
	char document_path[4000];
	int i = 0;

	app->get("data_path", data_path, NULL);

#ifdef WIN32
	fl_read_key(HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\"
		"Explorer\\Shell Folders",
		"Personal", document_path, 4000);
	win2unix(document_path);
#else
	get_home(document_path, 4000);
#endif

	while (!data_path) {
		char buf[1024];
		char *v;
		snprintf(buf, 1024, "%s/Gestion PME", document_path);
		v = fl_dir_chooser(_("R\303\251pertoire de travail"), buf, 0);
		if (!v || !*v) {
			i++;
			fl_alert(_("Vous devez choisir un r\303\251pertoire valide!"));
		} else {
			fl_make_path(v);
			if (fl_access(v, 0)) {
				v = NULL;
				i++;
				fl_alert(_("Impossible de cr\303\251er le r\303\251pertoire!"));	
			} else {
				data_path = strdup(v);
				app->set("data_path", data_path);
			}
		}
		if (i > 10) exit(-1);
	}

	app->get("tva_path", tva_path, NULL);
	if (!tva_path) {
		get("tva_path", tva_path, NULL);
		if (!tva_path) {
			snprintf(document_path, 1024, _("%s/Tva"), data_path);
			tva_path = strdup(document_path);
			set("tva_path", tva_path);
		}
		app->set("tva_path", tva_path);
	}

	app->get("article_path", article_path, NULL);
	if (!article_path) {
		get("article_path", article_path, NULL);
		if (!article_path) {
			snprintf(document_path, 1024, _("%s/Articles"), data_path);
			article_path = strdup(document_path);
			set("article_path", article_path);
		}
		app->set("article_path", article_path);
	}

	app->get("customer_path", customer_path, NULL);
	if (!customer_path) {
		get("customer_path", customer_path, NULL);
		if (!customer_path) {
			snprintf(document_path, 1024, _("%s/Clientes"), data_path);
			customer_path = strdup(document_path);
			set("customer_path", customer_path);
		}
		app->set("customer_path", customer_path);
	}
}

Settings::~Settings()
{
	free(tva_path);
	free(article_path);
	free(customer_path);
	free(data_path);
	delete(app);
}


//
// End of "$Id: $".
//

