//
// "$Id: $"
//
// Customer. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//


#include "TableInput.h"
#include <FL/fl_draw.H>


TableInput::TableInput(void) : Fl_Group (1, 1, 2, 2)
{
	hide();
	text_input = new TableInputText(1,1,2,2);
	current = text_input;
	end();
}

void TableInput::draw(void)
{
	Fl_Group::draw();	
}

void TableInput::resize(int X, int Y, int W, int H)
{
	Fl_Group::resize(X, Y, W, H);
}

void TableInput::value(const char *v)
{
	switch (type) {
	default:
		text_input->value(v);
	}	
}

const char *TableInput::value(void)
{
	switch (type) {
	default:
		return text_input->value();
	}
}

void TableInput::show(const char *ty)
{
	const char *val = value();
	current->hide();
	if (!ty) ty = "a";

	type = ty[0];
	switch (type) {

	default:
		text_input->value(value());
		current = text_input;
	}
	current->show();
	current->take_focus();
	Fl_Group::show();
}

void TableInput::run_callback(void)
{
	do_callback();
}

int TableInput::handle(int e)
{
	return current->handle(e);
	return Fl_Group::handle(e);
}

void TableInput::draw_selected(int X, int Y, int W, int H, 
	const char *v, const char *t)
{
	int a;

	fl_push_clip(X, Y, W, H);
	fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_YELLOW);
	fl_pop_clip();

	fl_push_clip(X+3, Y+3, W-6, H-6);
	fl_color(FL_BLACK);
	a = FL_ALIGN_WRAP|FL_ALIGN_LEFT|FL_ALIGN_TOP;
	fl_font(FL_HELVETICA, 14);
	if (v) fl_draw(v, X+3, Y+3, W-6, H-6, (Fl_Align)a, NULL, 0);
	fl_pop_clip();	
}

void TableInput::draw(int X, int Y, int W, int H, const char *v, const char *t)
{
	int a;

	fl_push_clip(X, Y, W, H);
	fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, FL_WHITE);
	fl_pop_clip();

	fl_push_clip(X+3, Y+3, W-6, H-6);
	fl_color(FL_BLACK);
	a = FL_ALIGN_WRAP|FL_ALIGN_LEFT|FL_ALIGN_TOP;
	fl_font(FL_HELVETICA, 14);
	if (v) fl_draw(v, X+3, Y+3, W-6, H-6, (Fl_Align)a, NULL, 0);
	fl_pop_clip();	
}

void TableInput::get_font(char *value, char *type, Fl_Font *fnt, 
		int *size, int *underline)
{
	*size = 12;
	*fnt = (Fl_Font)0;
	*underline = 0;
}	


TableInputText::TableInputText(int X, int Y, int W, int H) :
	Fl_Multiline_Input(X, Y, W, H)
{
	type(FL_MULTILINE_INPUT_WRAP);
}

int TableInputText::handle(int e)
{
	if (e == FL_KEYBOARD && Fl::e_length == 1 &&
		Fl::e_text[0] == '\r' &&
		!(Fl::event_state() & FL_ALT)) 
	{
		((TableInput*)parent())->run_callback();
		return 1;
	}
	return Fl_Multiline_Input::handle(e);
}


//
// End of "$Id: $".
//

