//
// "$Id: $"
//
// Application main object. 
//
//                   Copyright (c) 2004, O'ksi'D
//                      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of O'ksi'D nor the names of its contributors
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Author : Jean-Marc Lienher (http://www.oksid.ch)
//

#include <FL/x.H>
#include "TheApp.h"
#include "PrefDialog.h"
#include "xpm/save.xpm"
#include "xpm/print.xpm"
#include "xpm/open.xpm"
#include "xpm/new.xpm"
#include "xpm/preview.xpm"
#include "xpm/copy.xpm"
#include "xpm/paste.xpm"
#include "xpm/cut.xpm"
#include "xpm/undo.xpm"
#include "xpm/find.xpm"
#include "xpm/ok.xpm"
#include "xpm/no.xpm"
#include <FL/fl_ask.H>
#include <FL/Fl_File_Chooser.H>

#ifdef WIN32
#include "../visualc/resource.h"
#endif

TheApp *the_app;

enum MenuId {
	NONE,
	FI_OPEN,
	FI_NEW,
	FI_SAVE,
	FI_PRINT,
	ED_CUT,
	TO_OPTIONS,
	HE_ABOUT,
};

static void click_cb(Fl_Widget *w, void *d)
{
	TheApp *a = (TheApp*) w->parent();
	switch ((int)d) {
	case FI_OPEN:
		a->open();
		break;
	case FI_SAVE:
		a->save();
		break;
	case FI_PRINT:
		a->print();
		break;
	case ED_CUT:
	case TO_OPTIONS:
	case HE_ABOUT:
		break;
	}
}



TheApp::TheApp(int W, int H) : Fl_Window(W, H)
{

	the_app = this;

	resizable(this);
	printing = new Printing();
	settings = new Settings();
	settings->init();

	settings->app->get("margin_top", printing->ctx->margin_top,
			printing->ctx->margin_top);
	settings->app->get("margin_left", printing->ctx->margin_left,
			printing->ctx->margin_left);
	settings->app->get("margin_right", printing->ctx->margin_right,
			printing->ctx->margin_right);
	settings->app->get("margin_bottom", printing->ctx->margin_bottom,
			printing->ctx->margin_bottom);
	settings->app->get("page_width", printing->ctx->page_width,
			printing->ctx->page_width);
	settings->app->get("page_height", printing->ctx->page_height,
			printing->ctx->page_height);
	settings->app->get("landscape", printing->ctx->landscape,
			printing->ctx->landscape);

	icon((char *)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON1)));

	end();



	begin();
	box(FL_FLAT_BOX);
	Fl_Menu_Bar *m_menu = new Fl_Menu_Bar(0, 0, W, 28);
	m_menu->box(FL_THIN_UP_BOX);
	create_menu();
	m_menu->menu(mn_bar);


	b_res = new Fl_Button(W-1, H-81, 1, 1);
	b_res->hide();

	b_new = new Fl_Button(5, 30, 22, 22);
	b_new->callback(click_cb, (void*)FI_NEW);
	p_new = new Fl_Pixmap(new_xpm);
	p_new->label(b_new);

	b_open = new Fl_Button(28, 30, 22, 22);
	b_open->callback(click_cb, (void*)FI_OPEN);
	p_open = new Fl_Pixmap(open_xpm);
	p_open->label(b_open);

	b_save = new Fl_Button(51, 30, 22, 22);
	p_save = new Fl_Pixmap(save_xpm);
	p_save->label(b_save);
	b_save->callback(click_cb, (void*)FI_SAVE);

	b_print = new Fl_Button(80, 30, 22, 22);
	p_print =  new Fl_Pixmap(print_xpm);
	p_print->label(b_print);
	b_print->callback(click_cb, (void*)FI_PRINT);

	document = new Document(0, 55, W, H - 79);

	o_status = new Fl_Output(0, H - 22, W, 22);
	o_status->value(_("O'ksi'D Gestion PME ! V1.0.1"));
	o_status->color(FL_GRAY);
	o_status->box(FL_FLAT_BOX);

	resizable(b_res);
	size_range(200, 200);
	end();

}

TheApp::~TheApp()
{
	delete[] mn_file;
	delete[] mn_edit;
	delete[] mn_tools;
	delete[] mn_help;
	delete[] mn_bar;

	remove(document);
	delete(document);
	delete(settings);
}

void TheApp::create_menu()
{
	mn_file = new Fl_Menu_Item[4];
	mn_file->text = NULL;
	mn_file->add(_("Ouvrir"), FL_CTRL + 'o', click_cb, (void*)FI_OPEN, 0);
	
	mn_edit = new Fl_Menu_Item[4];
	mn_edit->text = NULL;
	mn_edit->add(_("Couper"), FL_CTRL + 'x', click_cb, (void*)ED_CUT, 0);
	
	mn_tools = new Fl_Menu_Item[4];
	mn_tools->text = NULL;
	mn_tools->add(_("Options"), 'o', click_cb, (void*)TO_OPTIONS, 0);	

	mn_help = new Fl_Menu_Item[4];
	mn_help->text = NULL;
	mn_help->add(_("\303\200 propos"), 'a', click_cb, (void*)HE_ABOUT, 0);

	mn_bar = new Fl_Menu_Item[5];
	mn_bar->text = NULL;
	mn_bar->add(_("Fichier"), FL_ALT + 'f', 0, (void*)mn_file, FL_SUBMENU_POINTER);
	mn_bar->add(_("Edition"), FL_ALT + 'e', 0, (void*)mn_edit, FL_SUBMENU_POINTER);
	mn_bar->add(_("Outils"), FL_ALT + 't', 0, (void*)mn_tools, FL_SUBMENU_POINTER);
	mn_bar->add(_("Aide"), FL_ALT + 'h', 0, (void*)mn_help, FL_SUBMENU_POINTER);
}

void TheApp::open()
{
	char *p;
	char *pa;

	pa = document->path();
	if (!pa) pa = settings->data_path;
	p = fl_file_chooser(_("Ouvrir / Importer fichier"), 
		_("Devis/Facture (*.{oki,oke})"), pa);
	if (!p) return;
	if (document->changed()) {
		if (fl_ask(_("Le document courant "
			"\303\240 \303\251t\303\251 modifi\303\251."
			" Faut'il le sauvegarder ?")))
		{
			save();
		}
	}
	remove(document);
	delete(document);
	begin();
	document = new Document(0, 55, w(), h() - 79, p);
	end();
}

void TheApp::save()
{
	document->save();
}

void TheApp::show(int ac, char **av)
{
	argv = av;
	argc = ac;
	Fl_Window::show(ac, av);
}

void TheApp::print()
{
	printing->preview(document);
}

//
// End of "$Id: $".
//

